/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
@file:Suppress("NOTHING_TO_INLINE")
@file:OptIn(ExperimentalContracts::class)

package dk.nsp.testassertions

import org.hamcrest.CoreMatchers.hasItem
import org.hamcrest.CoreMatchers.not
import org.hamcrest.MatcherAssert.assertThat
import org.junit.jupiter.api.Assertions.assertEquals
import org.junit.jupiter.api.Assertions.assertFalse
import org.junit.jupiter.api.Assertions.assertNotNull
import org.junit.jupiter.api.Assertions.assertNull
import org.junit.jupiter.api.Assertions.assertTrue
import kotlin.contracts.ExperimentalContracts
import kotlin.contracts.contract

/**
 * Test assertions helpers to enable assertions on the test target, instead of having test target as param.
 */

/**
 * Assert this is the given type
 * @receiver [Any] the receiver we are testing is the same type as [T]
 * @param message [String] the message to print if the receiver is a different type from [T]
 */

inline fun <reified T> Any.assertIs(
    message: String = "expected `$this` of type `${this::class.simpleName}`  to be of type `${T::class}`, but it is not",
) {
    contract {
        returns() implies (this@assertIs is T)
    }
    assertTrue(this is T, message)
}

inline fun <reified T> Any?.assertIsAndApply(
    reason: String = "`$this` was not of type `${T::class}`",
    letAction: T.() -> Unit,
) {
    assertNotNull()
    assertIs<T>(reason)
    this.let(letAction)
}

/**
 * Assert that this is the same type [otherValue] and that they are equal
 */
inline fun <reified T> Any?.assertAs(
    otherValue: T,
    message: String = "value of \"$this\" is not the expected \"$otherValue\"",
) {
    contract {
        returns() implies (this@assertAs is T)
    }

    assertNotNull()
    assertIs<T>()
    assertEquals(this as? T, otherValue)
}

inline fun <T> T.assertNull() = assertNull(this)

inline fun <T> T.assertNotNull() {
    contract {
        returns() implies (this@assertNotNull != null)
    }
    assertNotNull(this)
}

inline fun <reified T> T?.assertNotNullAndApply(
    letAction: T.() -> Unit,
) {
    assertNotNull()
    this.let(letAction)
}

/**
 * Asserts that this is equal to [expected] via [assertEquals]
 */
inline fun <T> T.assertEquals(expected: T, message: String? = "Was $this, but expected $expected") =
    assertEquals(expected, this, message)

inline fun <T> T.assertNotNullAndEquals(expected: T) {
    this.assertNotNull()
    this.assertEquals(expected)
}

/**
 * Asserts that this [List] does contain [item]
 */
inline fun <T> List<T>.assertContains(item: T, reason: String = "Should not contain $item") =
    assertThat(reason, this, hasItem(item))

//region Boolean expressions
inline fun Boolean.assertTrue(reason: String = "Should be true") = assertTrue(this, reason)
inline fun Boolean.assertFalse(reason: String = "Should be false") = assertFalse(this, reason)
//endregion

//region Lists
/**
 * Asserts that this [List] does not contain [item]
 */
inline fun <T> List<T>.assertNotContains(item: T, reason: String = "Should not contain $item") =
    assertThat(reason, this, not(hasItem(item)))
//endregion
