/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.api.auth

import android.net.Uri
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.tryOrFailed
import net.openid.appauth.AuthState
import net.openid.appauth.AuthorizationException
import net.openid.appauth.AuthorizationService
import net.openid.appauth.AuthorizationServiceConfiguration
import net.openid.appauth.TokenRequest
import net.openid.appauth.TokenResponse
import kotlin.coroutines.resume
import kotlin.coroutines.suspendCoroutine

/**
 * Helper extension function to map service discovery callback in a coroutine
 * and wrapping the result
 * @receiver AuthorizationService
 * @param forDiscoveryUrl Uri
 * @return ResultWrapper<AuthorizationServiceConfiguration>
 */
suspend fun getServiceConfiguration(forDiscoveryUrl: Uri): ResultWrapper<AuthorizationServiceConfiguration> =
    suspendCoroutine { continuation ->
        AuthorizationServiceConfiguration.fetchFromIssuer(forDiscoveryUrl) { serviceConfiguration: AuthorizationServiceConfiguration?, exception: AuthorizationException? ->
            continuation.resume(
                ResultWrapper.map(
                    serviceConfiguration,
                    exception
                        ?: AuthException.UnknownError("Something went wrong when discovering services"),
                ),
            )
        }
    }

/**
 * Maps [AuthorizationService.TokenResponseCallback] to a coroutine and wraps result in [ResultWrapper]
 * @receiver AuthorizationService
 * @param tokenRequest TokenRequest
 * @return ResultWrapper<TokenResponse>
 */
suspend fun AuthorizationService.performTokenRequest(tokenRequest: TokenRequest): ResultWrapper<TokenResponse> =
    suspendCoroutine { continuation ->
        performTokenRequest(tokenRequest) { response: TokenResponse?, exception: AuthorizationException? ->
            continuation.resume(
                ResultWrapper.map(
                    response,
                    exception ?: AuthException.UnknownError(
                        "Something went wrong with [authService.performTokenRequest]",
                    ),
                ),
            )
        }
    }

/**
 * Maps [AuthState.AuthStateAction] to a coroutine and wrapes result in [ResultWrapper]
 * @receiver AuthState
 * @return ResultWrapper<String>
 */
suspend fun AuthState.performActionWithFreshTokens(authService: AuthorizationService): ResultWrapper<String> =
    suspendCoroutine { continuation ->
        tryOrFailed {
            performActionWithFreshTokens(authService) { accessToken: String?, _: String?, exception: AuthorizationException? ->
                continuation.resume(
                    ResultWrapper.map(
                        accessToken,
                        exception ?: AuthException.UnknownError(
                            "Something went wrong while fetching accessToken",
                        ),
                    ),
                )
            }
        }
    }
