/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.api.auth.keyserver.models

import android.os.Parcelable
import android.util.Base64
import dk.nsp.shared.tryAndLog
import kotlinx.parcelize.Parcelize
import java.security.Key
import javax.crypto.Cipher
import javax.crypto.spec.IvParameterSpec
import javax.crypto.spec.SecretKeySpec

/**
 * @property key String the AES key
 * @property keyId String the key's id
 * @property longSecret String the longsecret working like a second password
 * @constructor
 */
@Parcelize
data class KeyModelDTO(
    val key: String,
    val keyId: String,
    val longSecret: String,
) : Parcelable

//region Crypto helpers
private const val aesCipher: String = "AES"
private const val aesCipherCbc5Padding = "$aesCipher/CBC/PKCS5Padding"

// An iv of null's / 0's /128 bit => 16 characters
private val iv: IvParameterSpec = IvParameterSpec(ByteArray(16) { 0 })

/**
 * Encrypts the given data with [KeyModelDTO.key]
 * @receiver KeyModel
 * @param data ByteArray
 * @return ByteArray?
 */
fun KeyModelDTO.encrypt(data: ByteArray): ByteArray? {
    val encryptionKey: Key = aesKey() ?: return null
    return tryAndLog {
        Cipher.getInstance(aesCipherCbc5Padding)?.let { cipher ->
            cipher.init(Cipher.ENCRYPT_MODE, encryptionKey, iv)
            cipher.doFinal(data)
        }
    }
}

/**
 * Decrypts the given content using [KeyModelDTO.key]
 * @receiver KeyModel
 * @param data ByteArray
 * @return ByteArray?
 */
fun KeyModelDTO.decrypt(data: ByteArray): ByteArray? {
    val decryptionKey: Key = aesKey() ?: return null
    return tryAndLog {
        Cipher.getInstance(aesCipherCbc5Padding)?.let { cipher ->
            cipher.init(Cipher.DECRYPT_MODE, decryptionKey, iv)
            cipher.doFinal(data)
        }
    }
}

/**
 * Unpacks the AES key for this [KeyModelDTO]
 * @receiver KeyModel
 * @return Key?
 */
private fun KeyModelDTO.aesKey(): Key? {
    // Decode the base 64 encoding
    val decodedKey = Base64.decode(key, Base64.DEFAULT) ?: return null
    return SecretKeySpec(decodedKey, aesCipher)
}
//endregion
