/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.api.configuration

import dk.nsp.api.util.EpisodeOfCareHeader
import dk.nsp.api.util.extensions.asyncIO
import dk.nsp.models.CheckListItem
import dk.nsp.models.DateOfBirth
import dk.nsp.shared.ApiCallCallFailed
import dk.nsp.shared.ApiCallCallSuccess
import dk.nsp.shared.Failed
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.Success
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Deferred
import java.time.LocalDateTime

class ConfigurationControllerImpl(private val api: ConfigurationApi) : IConfigurationController {

    override fun removeConsent(scope: CoroutineScope): Deferred<ResultWrapper<Unit>> =
        scope.asyncIO {
            when (val result = api.removeConsent()) {
                is ApiCallCallSuccess -> return@asyncIO Success(Unit)
                is ApiCallCallFailed -> return@asyncIO Failed(result.error)
            }
        }

    override fun updateChecklistItem(
        scope: CoroutineScope,
        episodeOfCareId: String,
        updateChecklistBody: CheckListItem,
    ): Deferred<ResultWrapper<CheckListItem>> =
        scope.asyncIO {
            val result = api.updateChecklistItem(
                episodeOfCareHeader = EpisodeOfCareHeader.createWith(episodeOfCareId),
                checklistId = updateChecklistBody.id.toString(), // DGAF-2224: Service doesn't work as expected, so here we should provide check list item id (like on iOS)
                checklistItem = updateChecklistBody,
            )
            return@asyncIO result.mapTo()
        }

    override fun getCheckedChecklistItems(
        scope: CoroutineScope,
        episodeOfCareId: String,
    ): Deferred<ResultWrapper<List<String>>> =
        scope.asyncIO {
            val result = api.getCheckedChecklistItems(
                EpisodeOfCareHeader.createWith(
                    episodeOfCareId,
                ),
            )
            return@asyncIO result.mapTo()
        }

    override fun getDateOfBirth(
        scope: CoroutineScope,
        episodeOfCareId: String,
    ): Deferred<ResultWrapper<DateOfBirth>> = scope.asyncIO {
        val result = api.getDateOfBirth(
            EpisodeOfCareHeader.createWith(
                episodeOfCareId,
            ),
        )
        return@asyncIO result.mapTo()
    }

    override fun updateDateOfBirth(
        scope: CoroutineScope,
        episodeOfCareId: String,
        newDate: LocalDateTime?,
    ): Deferred<ResultWrapper<DateOfBirth>> = scope.asyncIO {
        val result = api.updateDateOfBirth(
            EpisodeOfCareHeader.createWith(
                episodeOfCareId,
            ),
            DateOfBirth(newDate),
        )
        return@asyncIO result.mapTo()
    }
}
