/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.api.auth.keyserver

import dk.nsp.api.auth.keyserver.models.KeyModelDTO
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIsAndApply
import io.mockk.coEvery
import io.mockk.mockk
import kotlinx.coroutines.runBlocking
import org.junit.jupiter.api.Test

class KeyServerControllerImplTest {
    private val failingApi = mockk<KeyServerApi> {
        coEvery {
            createKey(any())
        } throws Exception("Failed to create key")

        coEvery {
            getKey(any(), any())
        } throws Exception("Failed to get key")

        coEvery {
            getKeyViaLongSecret(any(), any())
        } throws Exception("Failed to get key via long secret")
    }

    private val keyApi = mockk<KeyServerApi> {
        coEvery {
            createKey(any())
        } returns KeyModelDTO(
            "create-Key",
            "create-KeyId",
            "create-LongSecret",
        )

        coEvery {
            getKey(any(), any())
        } returns KeyModelDTO(
            "get-Key",
            "get-KeyId",
            "get-LongSecret",
        )

        coEvery {
            getKeyViaLongSecret(any(), any())
        } returns KeyModelDTO(
            "viaLong-key",
            "viaLong-keyId",
            "viaLong-longSecret",
        )
    }

    @Test
    fun createKey() = runBlocking {
        KeyServerController(keyApi)
            .createKey("")
            .assertIsAndApply<Success<KeyModelDTO>> {
                value.assertEquals(
                    KeyModelDTO(
                        "create-Key",
                        "create-KeyId",
                        "create-LongSecret",
                    ),
                )
            }

        KeyServerController(failingApi)
            .createKey("")
            .assertIsAndApply<Failed> {
                error.message.assertEquals("Failed to create key")
            }
    }

    @Test
    fun getKey() = runBlocking {
        KeyServerController(keyApi)
            .getKey("", "")
            .assertIsAndApply<Success<KeyModelDTO>> {
                value.assertEquals(
                    KeyModelDTO(
                        "get-Key",
                        "get-KeyId",
                        "get-LongSecret",
                    ),
                )
            }

        KeyServerController(failingApi)
            .getKey("", "")
            .assertIsAndApply<Failed> {
                error.message.assertEquals("Failed to get key")
            }
    }

    @Test
    fun getKeyViaLongSecret() = runBlocking {
        KeyServerController(keyApi)
            .getKeyViaLongSecret("", "")
            .assertIsAndApply<Success<KeyModelDTO>> {
                KeyModelDTO(
                    "viaLong-key",
                    "viaLong-keyId",
                    "viaLong-longSecret",
                )
            }

        KeyServerController(failingApi)
            .getKeyViaLongSecret("", "")
            .assertIsAndApply<Failed> {
                error.message.assertEquals("Failed to get key via long secret")
            }
    }

    class CompanionTest {

        @Test
        fun getInstance() {
            val instance = KeyServerController.get()
            val secondInstance = KeyServerController.get()

            // Ensure we aren't recreating and instance, but reusing
            secondInstance.assertEquals(instance)
        }
    }
}
