/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.api.cms

import dk.nsp.api.cms.models.ArticleCategoryDTO
import dk.nsp.api.cms.models.ChecklistDTO
import dk.nsp.api.cms.models.DictionaryEntryDTO
import dk.nsp.api.cms.models.FAQItemDTO
import dk.nsp.api.cms.models.PageDTO
import dk.nsp.api.cms.models.RecommendedArticlesDTO
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIsAndApply
import io.mockk.coEvery
import io.mockk.mockk
import kotlinx.coroutines.runBlocking
import org.junit.jupiter.api.Test
import java.time.LocalDateTime

class CMSControllerImplTest {

    class GetDictionary {
        private val failingApi = mockk<CMSApi> {
            coEvery {
                getDictionary()
            } throws Exception("Failed to fetch dictionary")
        }
        private val api = mockk<CMSApi> {
            coEvery {
                getDictionary()
            } returns listOf(
                DictionaryEntryDTO(
                    "title1",
                    "text1",
                ),
            )
        }

        @Test
        fun success() = runBlocking {
            CMSControllerImpl(api).getDictionary(this)
                .await()
                .assertIsAndApply<Success<List<DictionaryEntryDTO>>> {
                    value.size.assertEquals(1)
                    value.getOrNull(0).assertEquals(
                        DictionaryEntryDTO(
                            "title1",
                            "text1",
                        ),
                    )
                }
        }

        @Test
        fun failed() = runBlocking {
            CMSControllerImpl(failingApi)
                .getDictionary(this)
                .await()
                .assertIsAndApply<Failed> {
                    error.message.assertEquals("Failed to fetch dictionary")
                }
        }
    }

    class GetFAQ {
        private val failingApi = mockk<CMSApi> {
            coEvery {
                getFAQ()
            } throws Exception("Failed to get FAQ")
        }
        private val api = mockk<CMSApi> {
            coEvery {
                getFAQ()
            } returns listOf(
                FAQItemDTO(
                    "faqTitle",
                    "FAQText",
                    LocalDateTime.of(2020, 8, 7, 6, 5),
                ),
            )
        }

        @Test
        fun success() = runBlocking {
            CMSControllerImpl(api)
                .getFAQ(this)
                .await()
                .assertIsAndApply<Success<List<FAQItemDTO>>> {
                    value.size.assertEquals(1)
                    value.getOrNull(0).assertEquals(
                        FAQItemDTO(
                            "faqTitle",
                            "FAQText",
                            LocalDateTime.of(2020, 8, 7, 6, 5),
                        ),
                    )
                }
        }

        @Test
        fun failed() = runBlocking {
            CMSControllerImpl(failingApi)
                .getFAQ(this)
                .await()
                .assertIsAndApply<Failed> {
                    error.message.assertEquals("Failed to get FAQ")
                }
        }
    }

    class GetArticleSections {
        private val failingApi = mockk<CMSApi> {
            coEvery {
                getArticles()
            } throws Exception("Failed to get article sections")
        }
        private val api = mockk<CMSApi> {
            coEvery {
                getArticles()
            } returns listOf(
                ArticleCategoryDTO(
                    10,
                    "ACName",
                    "ACDescription",
                    "ACIcon",
                    emptyList(),
                    "ACSlug",
                ),
            )
        }

        @Test
        fun success() = runBlocking {
            CMSControllerImpl(api)
                .getArticleSections(this)
                .await()
                .assertIsAndApply<Success<List<ArticleCategoryDTO>>> {
                    value.size.assertEquals(1)
                    value.getOrNull(0).assertEquals(
                        ArticleCategoryDTO(
                            10,
                            "ACName",
                            "ACDescription",
                            "ACIcon",
                            emptyList(),
                            "ACSlug",
                        ),
                    )
                }
        }

        @Test
        fun failed() = runBlocking {
            CMSControllerImpl(failingApi)
                .getArticleSections(this)
                .await()
                .assertIsAndApply<Failed> {
                    error.message.assertEquals("Failed to get article sections")
                }
        }
    }

    class GetArticleCategories {
        private val failingApi = mockk<CMSApi> {
            coEvery {
                getArticleCategories(any())
            } throws Exception("Failed to get article categories")
        }
        private val api = mockk<CMSApi> {
            coEvery {
                getArticleCategories(any())
            } returns RecommendedArticlesDTO(
                currentWeek = listOf(
                    ArticleCategoryDTO(
                        categoryId = 10,
                        name = "ACName",
                        description = "ACDescription",
                        icon = "ACIcon",
                        articles = emptyList(),
                        slug = "ACSlug",
                    ),
                ),
                nextWeek = emptyList(),
            )
        }

        @Test
        fun success() = runBlocking {
            CMSControllerImpl(api)
                .getArticleCategories(this, 1)
                .await()
                .assertIsAndApply<Success<RecommendedArticlesDTO>> {
                    value.currentWeek.size.assertEquals(1)
                    value.currentWeek.getOrNull(0).assertEquals(
                        ArticleCategoryDTO(
                            categoryId = 10,
                            name = "ACName",
                            description = "ACDescription",
                            icon = "ACIcon",
                            articles = emptyList(),
                            slug = "ACSlug",
                        ),
                    )
                }
        }

        @Test
        fun failed() = runBlocking {
            CMSControllerImpl(failingApi)
                .getArticleCategories(this, 1)
                .await()
                .assertIsAndApply<Failed> {
                    error.message.assertEquals("Failed to get article categories")
                }
        }
    }

    class GetAfterBirthArticleCategories {
        private val failingApi = mockk<CMSApi> {
            coEvery {
                getAfterBirthArticleCategories(any())
            } throws Exception("Failed to get after birth article categories")
        }
        private val api = mockk<CMSApi> {
            coEvery {
                getAfterBirthArticleCategories(any())
            } returns RecommendedArticlesDTO(
                currentWeek = listOf(
                    ArticleCategoryDTO(
                        categoryId = 10,
                        name = "ACName",
                        description = "ACDescription",
                        icon = "ACIcon",
                        articles = emptyList(),
                        slug = "ACSlug",
                    ),
                ),
                nextWeek = emptyList(),
            )
        }

        @Test
        fun success() = runBlocking {
            CMSControllerImpl(api)
                .getAfterBirthArticleCategories(this, 1)
                .await()
                .assertIsAndApply<Success<RecommendedArticlesDTO>> {
                    value.currentWeek.size.assertEquals(1)
                    value.currentWeek.getOrNull(0).assertEquals(
                        ArticleCategoryDTO(
                            categoryId = 10,
                            name = "ACName",
                            description = "ACDescription",
                            icon = "ACIcon",
                            articles = emptyList(),
                            slug = "ACSlug",
                        ),
                    )
                }
        }

        @Test
        fun failed() = runBlocking {
            CMSControllerImpl(failingApi)
                .getAfterBirthArticleCategories(this, 1)
                .await()
                .assertIsAndApply<Failed> {
                    error.message.assertEquals("Failed to get after birth article categories")
                }
        }
    }

    class SearchArticles {
        private val failingApi = mockk<CMSApi> {
            coEvery {
                searchArticles(any())
            } throws Exception("Failed to search articles")
        }
        private val api = mockk<CMSApi> {
            coEvery {
                searchArticles(any())
            } returns listOf(
                ArticleCategoryDTO(
                    10,
                    "ACName",
                    "ACDescription",
                    "ACIcon",
                    emptyList(),
                    "ACSlug",
                ),
            )
        }

        @Test
        fun success() = runBlocking {
            CMSControllerImpl(api)
                .searchArticles(this, "searchQuery")
                .await()
                .assertIsAndApply<Success<List<ArticleCategoryDTO>>> {
                    value.size.assertEquals(1)
                    value.getOrNull(0).assertEquals(
                        ArticleCategoryDTO(
                            10,
                            "ACName",
                            "ACDescription",
                            "ACIcon",
                            emptyList(),
                            "ACSlug",
                        ),
                    )
                }
        }

        @Test
        fun failed() = runBlocking {
            CMSControllerImpl(failingApi)
                .searchArticles(this, "searchString")
                .await()
                .assertIsAndApply<Failed> {
                    error.message.assertEquals("Failed to search articles")
                }
        }
    }

    class GetChecklists {
        private val failingApi = mockk<CMSApi> {
            coEvery {
                getChecklists()
            } throws Exception("Failed to get checklists")
        }
        private val api = mockk<CMSApi> {
            coEvery {
                getChecklists()
            } returns listOf(
                ChecklistDTO(
                    10,
                    "ChecklistTitle",
                    "ChecklistText",
                    emptyList(),
                ),
            )
        }

        @Test
        fun success() = runBlocking {
            CMSControllerImpl(api)
                .getChecklists(this)
                .await()
                .assertIsAndApply<Success<List<ChecklistDTO>>> {
                    value.size.assertEquals(1)
                    value.getOrNull(0).assertEquals(
                        ChecklistDTO(
                            10,
                            "ChecklistTitle",
                            "ChecklistText",
                            emptyList(),
                        ),
                    )
                }
        }

        @Test
        fun failed() = runBlocking {
            CMSControllerImpl(failingApi)
                .getChecklists(this)
                .await()
                .assertIsAndApply<Failed> {
                    error.message.assertEquals("Failed to get checklists")
                }
        }
    }

    class GetPageContent {
        private val failingApi = mockk<CMSApi> {
            coEvery {
                getPageContentFor(any())
            } throws Exception("Failed to get page content")
        }
        private val api = mockk<CMSApi> {
            coEvery {
                getPageContentFor(any())
            } returns PageDTO(
                "Content",
            )
        }

        @Test
        fun success() = runBlocking {
            CMSControllerImpl(api)
                .getPageContent(this, "content")
                .await()
                .assertIsAndApply<Success<PageDTO>> {
                    value.assertEquals(
                        PageDTO(
                            "Content",
                        ),
                    )
                }
        }

        @Test
        fun failed() = runBlocking {
            CMSControllerImpl(failingApi)
                .getPageContent(this, "content")
                .await()
                .assertIsAndApply<Failed> {
                    error.message.assertEquals("Failed to get page content")
                }
        }
    }
}
