/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.api.configuration

import dk.nsp.api.util.EpisodeOfCareHeader
import dk.nsp.models.CheckListItem
import dk.nsp.models.DateOfBirth
import dk.nsp.shared.ApiCallCallFailed
import dk.nsp.shared.ApiCallCallSuccess
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIs
import dk.nsp.testassertions.assertIsAndApply
import io.mockk.coEvery
import io.mockk.mockk
import kotlinx.coroutines.runBlocking
import org.junit.jupiter.api.Test
import java.time.LocalDateTime

class ConfigurationControllerImplTest {

    class RemoveConsent {
        @Test
        fun removeConsentSuccess() = runBlocking {
            successController.removeConsent(this).await()
                .assertIs<Success<Unit>>()
        }

        @Test
        fun removeConsentFailed() = runBlocking {
            failingController.removeConsent(this)
                .await()
                .assertIsAndApply<Failed> {
                    this.error.message.assertEquals("Failed to remove consent")
                }
        }
    }

    class UpdateChecklistItem {
        @Test
        fun updateChecklistItemSuccess() = runBlocking {
            successController.updateChecklistItem(
                this,
                "validId",
                CheckListItem(
                    "id1",
                    true,
                ),
            ).await()
                .assertIsAndApply<Success<CheckListItem>> {
                    value.assertEquals(
                        CheckListItem(
                            "id1",
                            true,
                        ),
                    )
                }
        }
    }

    class GetCheckedChecklistItems {
        @Test
        fun getCheckedChecklistItemsSuccess() = runBlocking {
            successController.getCheckedChecklistItems(
                this,
                "validId",
            ).await().assertIsAndApply<Success<List<String>>> {
                value.size.assertEquals(2)
                value.getOrNull(0).assertEquals("yes")
                value.getOrNull(1).assertEquals("no")
            }
        }

        @Test
        fun getCheckedChecklistItemsFailed() = runBlocking {
            failingController.getCheckedChecklistItems(this, "")
                .await()
                .assertIsAndApply<Failed> {
                    this.error.message.assertEquals("Failed to getCheckedChecklistItems")
                }
        }
    }

    class GetDateOfBirth {
        @Test
        fun getDateOfBirthSuccess() = runBlocking {
            successController.getDateOfBirth(
                this,
                "validId",
            ).await().assertIsAndApply<Success<DateOfBirth>> {
                value.assertEquals(
                    DateOfBirth(LocalDateTime.of(2021, 1, 2, 3, 4)),
                )
            }
        }

        @Test
        fun getDateOfBirthFailed() = runBlocking {
            failingController.getDateOfBirth(this, "")
                .await()
                .assertIsAndApply<Failed> {
                    this.error.message.assertEquals("Failed to get DateOfBirth")
                }
        }
    }

    class UpdateDateOfBirth {
        @Test
        fun updateDateOfBirthSuccess() = runBlocking {
            val input = LocalDateTime.of(2021, 4, 3, 2, 1)
            successController.updateDateOfBirth(
                this,
                "validId",
                input,
            ).await()
                .assertIsAndApply<Success<DateOfBirth>> {
                    value.assertEquals(
                        DateOfBirth(input),
                    )
                }
        }

        @Test
        fun updateDateOfBirthFailed() = runBlocking {
            failingController.updateDateOfBirth(this, "", null)
                .await()
                .assertIsAndApply<Failed> {
                    this.error.message.assertEquals("Failed to set DateOfBirth")
                }
        }
    }

    companion object {
        private val failingApi by lazy {
            mockk<ConfigurationApi> {
                // Remove consent
                coEvery {
                    removeConsent()
                } returns ApiCallCallFailed(Exception("Failed to remove consent"))

                // UpdateCheckListItem
                coEvery {
                    getCheckedChecklistItems(
                        EpisodeOfCareHeader.createWith(""),
                    )
                } returns ApiCallCallFailed(Exception("Failed to getCheckedChecklistItems"))

                // GetCheckedListItems
                coEvery {
                    updateChecklistItem(
                        EpisodeOfCareHeader.createWith(""),
                        "",
                        CheckListItem(),
                    )
                } returns ApiCallCallFailed(Exception("Failed to updateChecklistItem"))

                // GetDateOfBirth
                coEvery {
                    getDateOfBirth(EpisodeOfCareHeader.createWith(""))
                } returns ApiCallCallFailed(Exception("Failed to get DateOfBirth"))

                // UpdateDateOfBirth
                coEvery {
                    updateDateOfBirth(
                        EpisodeOfCareHeader.createWith(""),
                        any(),
                    )
                } returns ApiCallCallFailed(Exception("Failed to set DateOfBirth"))
            }
        }

        private val mockedApi by lazy {
            mockk<ConfigurationApi> {
                // Remove consent
                coEvery {
                    removeConsent()
                } returns ApiCallCallSuccess(Unit)

                // UpdateCheckListItem
                coEvery {
                    getCheckedChecklistItems(
                        EpisodeOfCareHeader.createWith("validId"),
                    )
                } returns ApiCallCallSuccess(listOf("yes", "no"))

                // GetCheckedListItems
                coEvery {
                    updateChecklistItem(
                        EpisodeOfCareHeader.createWith("validId"),
                        any(),
                        CheckListItem(
                            "id1",
                            true,
                        ),
                    )
                } returns ApiCallCallSuccess(
                    CheckListItem(
                        "id1",
                        true,
                    ),
                )

                // GetDateOfBirth
                coEvery {
                    getDateOfBirth(
                        EpisodeOfCareHeader.createWith("validId"),
                    )
                } returns ApiCallCallSuccess(DateOfBirth(LocalDateTime.of(2021, 1, 2, 3, 4)))

                // UpdateDateOfBirth
                coEvery {
                    updateDateOfBirth(
                        EpisodeOfCareHeader.createWith("validId"),
                        any(),
                    )
                } returns ApiCallCallSuccess(DateOfBirth(LocalDateTime.of(2021, 4, 3, 2, 1)))
            }
        }

        private val failingController by lazy {
            ConfigurationControllerImpl(failingApi)
        }

        private val successController by lazy {
            ConfigurationControllerImpl(mockedApi)
        }
    }
}
