/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.api.util

import dk.nsp.testassertions.assertFalse
import dk.nsp.testassertions.assertTrue
import net.openid.appauth.AuthorizationException
import net.openid.appauth.AuthorizationException.TYPE_OAUTH_TOKEN_ERROR
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.ResponseBody.Companion.toResponseBody
import org.junit.jupiter.api.Test
import retrofit2.HttpException
import retrofit2.Response

class NetworkErrorsBllTest {

    private fun getBasicHttpException(code: Int) = HttpException(
        Response.error<String>(
            code,
            "emptyBody"
                .toResponseBody(
                    "text/plain"
                        .toMediaType(),
                ),
        ),
    )

    @Test
    fun throwableNetworkErrorsApiIsHttpErrorWithCode() {
        Exception("").networkErrorsApi
            .isHttpErrorWithCode(200)
            .assertFalse()

        getBasicHttpException(404)
            .networkErrorsApi
            .isHttpErrorWithCode(404)
            .assertTrue()

        getBasicHttpException(500)
            .networkErrorsApi
            .isHttpErrorWithCode(404)
            .assertFalse()
    }

    @Test
    fun throwableNetworkErrorsApiIsRefreshTokenExpired() {
        AuthorizationException(
            TYPE_OAUTH_TOKEN_ERROR,
            1,
            "Auth error: invalid_grant",
            "Auth error: refresh token expired",
            null,
            null,
        ).networkErrorsApi
            .isRefreshTokenExpired()
            .assertTrue()

        AuthorizationException(
            TYPE_OAUTH_TOKEN_ERROR,
            1,
            null,
            null,
            null,
            null,
        ).networkErrorsApi
            .isRefreshTokenExpired()
            .assertFalse()

        getBasicHttpException(401)
            .networkErrorsApi
            .isRefreshTokenExpired()
            .assertFalse()
    }

    class AuthorizationExceptionIsRefreshTokenExpired {
        @Test
        fun isExpired() {
            AuthorizationException(
                TYPE_OAUTH_TOKEN_ERROR,
                1,
                "Auth error: invalid_grant",
                "Auth error: refresh token expired",
                null,
                null,
            ).isRefreshTokenExpired()
                .assertTrue()

            AuthorizationException(
                TYPE_OAUTH_TOKEN_ERROR,
                1,
                null,
                "Auth error: refresh token expired",
                null,
                null,
            ).isRefreshTokenExpired()
                .assertTrue()

            AuthorizationException(
                TYPE_OAUTH_TOKEN_ERROR,
                1,
                "Auth error: invalid_grant",
                null,
                null,
                null,
            ).isRefreshTokenExpired()
                .assertTrue()
        }

        @Test
        fun isNotExpiredError() {
            AuthorizationException(
                TYPE_OAUTH_TOKEN_ERROR,
                1,
                null,
                "Auth error: access token expired",
                null,
                null,
            ).isRefreshTokenExpired()
                .assertFalse()

            AuthorizationException(
                TYPE_OAUTH_TOKEN_ERROR,
                1,
                "Auth error: valid_grant",
                null,
                null,
                null,
            ).isRefreshTokenExpired()
                .assertFalse()
        }
    }
}
