/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.journal.sections.sectiondetails.aroundthepregnancy

import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.ui.main.journal.extensions.toTextModel
import dk.nsp.mingraviditet.util.extensions.toTextModel
import dk.nsp.mingraviditet.util.extensions.toTextModelOrDefault
import dk.nsp.models.ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod
import dk.nsp.models.ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriodMenstrualCycle
import dk.nsp.models.WeeklyAlcoholUnitsConsumption
import dk.nsp.models.WeeklyConsumptionDuringPregnancy
import dk.nsp.models.WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk
import dk.nsp.shared.util.extensions.toMediumFormat
import dk.nsp.util.textmodel.TextModel
import dk.nsp.util.toTextModel

fun ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod?.lastFirstDayOfCycleToTextModel(): TextModel =
    this?.lastPeriod.toMediumFormat().toTextModelOrDefault()

fun ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod?.getCalculatedExpectedDeliveryDate(): TextModel {
    return this?.expectedDateOfDelivery
        .toMediumFormat()
        .toTextModelOrDefault()
}

fun ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriodMenstrualCycle?.toTextModel(): TextModel {
    val minVal = this?.min?.value?.toTextModel()
    val maxVal = this?.max?.value?.toTextModel()

    return when {
        minVal != null && maxVal != null -> TextModel.from(
            listOf(minVal, maxVal),
            seperator = TextModel.from(" - "),
        )
        minVal != null -> minVal
        maxVal != null -> maxVal
        else -> TextModel.from("")
    }
}

fun WeeklyAlcoholUnitsConsumption?.getConsumption(): TextModel {
    val intValue = this?.consumption?.value ?: 0

    val valueTextModel = intValue.toTextModel()
    val unit = this?.consumption?.unit?.toTextModel()

    return if (unit == null) {
        TextModel.from(R.plurals.alcohol_units, intValue, valueTextModel)
    } else {
        TextModel.from(listOf(valueTextModel, unit), seperator = TextModel.from(" "))
    }
}

fun WeeklyConsumptionDuringPregnancy?.getAverageConsumption(): TextModel {
    val intValue = this?.consumption?.value ?: 0

    val valueTextModel = intValue.toTextModel()
    val unit = this?.consumption?.unit?.toTextModel()

    return if (unit == null) {
        TextModel.from(R.plurals.alcohol_units, intValue, valueTextModel)
    } else {
        TextModel.from(listOf(valueTextModel, unit), seperator = TextModel.from(" "))
    }
}

fun WeeklyConsumptionDuringPregnancy?.getExcessiveDrinkingCount(): TextModel {
    val excessiveDrinkingCount = this?.excessiveDrinkingCount?.toInt() ?: 0
    val value = excessiveDrinkingCount.toTextModel()

    return TextModel.from(R.plurals.excessive_drinking, excessiveDrinkingCount, value)
}

fun WeeklyConsumptionDuringPregnancy?.getExcessiveDrinkingWeeks(): TextModel {
    val excessiveDrinkingWeeksCommaSeparated =
        this?.excessiveDrinkingWeeks?.joinToString(separator = ", ") { it.toString() }

    return excessiveDrinkingWeeksCommaSeparated.toTextModelOrDefault()
}

fun WeeklyConsumptionDuringPregnancy?.getTweakPoints(): TextModel {
    return this?.tweakPoints?.value.toTextModelOrDefault()
}

fun WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk?.getExposureDuration(): TextModel {
    val start = this?.start.toMediumFormat()?.toTextModel()
    val end = this?.end.toMediumFormat()?.toTextModel()

    val models = when {
        start != null && end != null -> {
            listOf(start, end)
        }
        start != null -> {
            listOf(start, TextModel.from(""))
        }
        end != null -> {
            listOf(TextModel.from(""), end)
        }
        else -> null
    }

    return if (models != null) {
        TextModel.from(models, TextModel.from(" - "))
    } else {
        TextModel.from(R.string.common_missing_data)
    }
}
