/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge

import android.app.Activity
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.WindowInsets
import android.view.inputmethod.InputMethodManager
import androidx.core.widget.doOnTextChanged
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import com.commonsense.android.kotlin.views.extensions.scrollToTop
import com.google.android.material.snackbar.Snackbar
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.FragmentKnowledgeBinding
import dk.nsp.mingraviditet.domain.InfoBoxModel
import dk.nsp.mingraviditet.ui.main.common.DataCardStateViewModel
import dk.nsp.mingraviditet.ui.widgets.BottomInfoToggler
import dk.nsp.mingraviditet.util.android.recycler.DelegatingListAdapter
import dk.nsp.mingraviditet.util.extensions.handleToolbarState
import dk.nsp.mingraviditet.util.extensions.navigateSafe
import dk.nsp.mingraviditet.util.extensions.observeIfNotHandled
import dk.nsp.mingraviditet.util.extensions.setupToolbarStateListener

@AndroidEntryPoint
class KnowledgeFragment : Fragment() {
    private lateinit var binding: FragmentKnowledgeBinding

    private val knowledgeViewModel: KnowledgeViewModel by viewModels()
    private val infoBoxViewModel: DataCardStateViewModel by viewModels()

    private val adapter by lazy {
        DelegatingListAdapter(viewLifecycleOwner)
    }

    private var infoToggler: BottomInfoToggler? = null

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View {
        binding = FragmentKnowledgeBinding.inflate(inflater, container, false)
        binding.lifecycleOwner = viewLifecycleOwner
        binding.toolbarTitle.setText(R.string.bottom_navigation_title_knowledge)

        binding.recyclerview.adapter = adapter

        knowledgeViewModel.isRefreshing.observe(viewLifecycleOwner) {
            binding.swipeRefresh.isRefreshing = it
        }

        binding.swipeRefresh.setOnRefreshListener {
            knowledgeViewModel.onRefreshData()
        }

        knowledgeViewModel.onChecklistClicked.observeIfNotHandled(viewLifecycleOwner) {
            findNavController().navigateSafe(
                KnowledgeFragmentDirections.actionKnowledgeFragmentToChecklistDetailsFragment(it.id),
            )
        }

        knowledgeViewModel.onDataCombined.observe(viewLifecycleOwner) {
            if (knowledgeViewModel.isShowingSearch) return@observe

            adapter.submitList(it) {
                binding.recyclerview.scrollToTop(false)
                binding.recyclerview.scheduleLayoutAnimation()
            }
        }

        knowledgeViewModel.onError.observeIfNotHandled(viewLifecycleOwner) {
            Snackbar.make(binding.root, it, Snackbar.LENGTH_LONG)
                .setAction(R.string.common_ok) {}
                .show()
        }

        knowledgeViewModel.onArticleCategoryClicked.observeIfNotHandled(viewLifecycleOwner) {
            findNavController().navigateSafe(
                KnowledgeFragmentDirections.actionKnowledgeFragmentToKnowledgeArticlesFragment(
                    it,
                ),
            )
        }

        binding.searchBar.doOnTextChanged { text, _, _, _ ->
            text?.toString()?.let { searchTerm ->
                this.binding.swipeRefresh.isEnabled = searchTerm.isEmpty()
                knowledgeViewModel.searchFor(searchTerm)
            }
        }

        knowledgeViewModel.searchResults.observe(viewLifecycleOwner) {
            if (!knowledgeViewModel.isShowingSearch) return@observe

            adapter.submitList(it)
        }

        knowledgeViewModel.onSearchArticleClicked.observeIfNotHandled(viewLifecycleOwner) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                binding.root.windowInsetsController?.hide(WindowInsets.Type.ime())
            } else {
                val inputMethodManager =
                    context?.getSystemService(Activity.INPUT_METHOD_SERVICE) as InputMethodManager
                inputMethodManager.hideSoftInputFromWindow(binding.root.windowToken, 0)
            }
            findNavController().navigateSafe(
                KnowledgeFragmentDirections.actionKnowledgeFragmentToKnowledgeArticleFragment(
                    it.first,
                    it.second,
                ),
            )
        }

        infoToggler = BottomInfoToggler(binding.bottomInfo.bottomInfo, binding.recyclerview)
        infoBoxViewModel.infoBox.observe(viewLifecycleOwner) {
            if (it is InfoBoxModel.Orange) {
                binding.bottomInfo.setBottomInfo(it)
            } else {
                binding.bottomInfo.setBottomInfo(null)
            }
        }

        setupToolbarState()

        return binding.root
    }

    private fun setupToolbarState() {
        setupToolbarStateListener(binding.motionLayout) { newState ->
            knowledgeViewModel.toolbarState = newState
            binding.motionLayout.handleToolbarState(newState)
        }

        binding.motionLayout.handleToolbarState(knowledgeViewModel.toolbarState)
    }
}
