/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.article

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.drawable.TransitionDrawable
import android.view.View
import android.view.ViewGroup
import android.webkit.WebChromeClient
import androidx.core.view.updateLayoutParams
import com.commonsense.android.kotlin.base.extensions.coroutines.launchDefault
import com.commonsense.android.kotlin.views.extensions.gone
import com.commonsense.android.kotlin.views.extensions.visible
import dk.nsp.domain.articles.ArticleContentSection
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.ArticleParagraphCardRowBinding
import dk.nsp.mingraviditet.extensions.disable
import dk.nsp.mingraviditet.util.android.OpenExternalLinksInBrowserWebViewClient
import dk.nsp.mingraviditet.util.android.WebViewHelper
import dk.nsp.mingraviditet.util.android.WebViewHelper.setUpDarkModeSupport
import dk.nsp.mingraviditet.util.android.recycler.BindingViewHolder
import dk.nsp.mingraviditet.util.android.recycler.DelegationListAdapterInflater
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay

interface WebChromeClientDelegate {
    fun onShowCustomView(view: View, callback: WebChromeClient.CustomViewCallback)
    fun onHideCustomView()
}

class ArticleParagraphCardRow(
    val model: ArticleContentSection,
    val relevantId: Int,
    private val webChromeClientDelegate: WebChromeClientDelegate,
) :
    RecyclerAdapterView<ArticleParagraphCardRowBinding> {

    private var collapsedHeight: Int? = null

    @SuppressLint("SetJavaScriptEnabled", "ClickableViewAccessibility")
    override fun bindData(viewHolder: BindingViewHolder<ArticleParagraphCardRowBinding>) {
        with(viewHolder.binding) {
            // Ensure initial state is correct, before handling new data
            updateViewExpandedState(this)

            title.text = model.title
            webview.gone()
            progressBar.visible()
            collapsedGradient.gone()
            webview.webChromeClient = WebChromeClientImpl(webChromeClientDelegate)
            webview.webViewClient =
                OpenExternalLinksInBrowserWebViewClient(
                    root.context,
                    onPageFinished = {
                        webview.post {
                            updateViewExpandedState(viewHolder.binding)

                            progressBar.gone()
                            webview.visible()
                        }
                    },
                )

            webview.apply {
                setUpDarkModeSupport()

                // Enable js to be able to show videoes
                settings.javaScriptEnabled = true
                settings.useWideViewPort = false
                settings.loadWithOverviewMode = false

                // Disable scroll on touch
                disable()

                val htmlToLoad = WebViewHelper.createHtmlWithStyleSheet(
                    model.content,
                    root.context,
                )

                loadDataWithBaseURL(
                    "file:///android_asset/",
                    htmlToLoad,
                    "text/html",
                    "UTF-8",
                    "",
                )
            }

            // Show small animation when coming from search
            if (relevantId == model.id) {
                val transition =
                    rowBackground.background as TransitionDrawable

                CoroutineScope(Dispatchers.Main).launchDefault {
                    transition.startTransition(250)
                    delay(500)
                    transition.reverseTransition(500)
                }
            }

            expandCollapseButton.setOnClickListener {
                toggleExpandCollapseState(viewHolder.binding)
            }
        }
    }

    private fun toggleExpandCollapseState(binding: ArticleParagraphCardRowBinding) {
        model.isExpanded = !model.isExpanded
        updateViewExpandedState(binding)
    }

    private fun updateViewExpandedState(binding: ArticleParagraphCardRowBinding) {
        if (model.isExpanded) {
            binding.webview.updateLayoutParams {
                height = ViewGroup.LayoutParams.WRAP_CONTENT
            }
            binding.expandCollapseButton.setText(R.string.common_cta_collapse)
            binding.collapsedGradient.gone()
        } else {
            binding.webview.updateLayoutParams {
                height = getCollapsedHeight(binding.root.context)
            }
            binding.expandCollapseButton.setText(R.string.common_cta_expand)
            binding.collapsedGradient.visible()
        }
    }

    override val createHolderFunction: DelegationListAdapterInflater<ArticleParagraphCardRowBinding> =
        ArticleParagraphCardRowBinding::inflate

    override fun isContentTheSameAsInternal(other: RecyclerAdapterView<*>): Boolean =
        when (other) {
            is ArticleParagraphCardRow -> other.model == model && other.relevantId == relevantId
            else -> false
        }

    /**
     * Convenience to storing collapsed height, instead of resolving from resources each time
     */
    private fun getCollapsedHeight(context: Context) = collapsedHeight
        ?: context.resources.getDimensionPixelSize(R.dimen.article_content_collapsed_height)
            .also {
                collapsedHeight = it
            }

    private inner class WebChromeClientImpl(val webChromeClientDelegate: WebChromeClientDelegate) : WebChromeClient() {
        override fun onShowCustomView(view: View, callback: CustomViewCallback) {
            webChromeClientDelegate.onShowCustomView(view, callback)
        }

        override fun onHideCustomView() {
            webChromeClientDelegate.onHideCustomView()
        }
    }
}
