/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.article

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.webkit.WebChromeClient
import android.widget.FrameLayout
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.navArgs
import androidx.recyclerview.widget.LinearLayoutManager
import com.commonsense.android.kotlin.system.extensions.onBackPressed
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.databinding.FragmentKnowledgeArticleBinding
import dk.nsp.mingraviditet.util.android.recycler.DelegatingListAdapter
import dk.nsp.mingraviditet.util.extensions.observeIfNotHandled

@AndroidEntryPoint
class KnowledgeArticleFragment : Fragment() {

    private val adapter by lazy {
        DelegatingListAdapter(viewLifecycleOwner)
    }

    private val viewModel by viewModels<KnowledgeArticleViewModel>()

    private val args: KnowledgeArticleFragmentArgs by navArgs()

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View {
        val binding =
            FragmentKnowledgeArticleBinding.inflate(inflater, container, false)
        binding.lifecycleOwner = viewLifecycleOwner
        binding.toolbar.setNavigationOnClickListener { onBackPressed() }
        binding.toolbar.title = args.article.title

        binding.recyclerView.adapter = adapter

        viewModel.rowsToShow.observe(viewLifecycleOwner) {
            adapter.submitList(it)
        }

        viewModel.scrollToItem.observeIfNotHandled(viewLifecycleOwner) { position ->
            (binding.recyclerView.layoutManager as? LinearLayoutManager)?.scrollToPositionWithOffset(
                position,
                // Scroll the view to the top
                0,
            )
        }

        val fullScreenToggler = FullScreenToggler()
        viewModel.enterFullScreen.observe(viewLifecycleOwner) { singleEvent ->
            singleEvent.getIfNotHandled()?.let {
                fullScreenToggler.shownViewInFullScreen(it.view, it.callback)
            }
        }

        viewModel.leaveFullScreen.observe(viewLifecycleOwner) { singleEvent ->
            singleEvent.getIfNotHandled()?.let {
                fullScreenToggler.hideViewFromFullScreen()
            }
        }

        return binding.root
    }

    // Class that support showing a view in full screen. Used for the ChromeWebViewClient used by some of the row data
    // shown in this fragment.
    private inner class FullScreenToggler {
        private var view: View? = null
        private var originalSystemUIVisibility: Int = 0
        private var originalOrientation: Int = 0
        private var customViewCallback: WebChromeClient.CustomViewCallback? = null

        fun shownViewInFullScreen(view: View, callback: WebChromeClient.CustomViewCallback) {
            if (this.view != null) {
                hideViewFromFullScreen()
                return
            }

            this.view = view

            val activity = requireActivity()
            val window = activity.window
            val decorView = window.decorView as FrameLayout

            originalSystemUIVisibility = decorView.systemUiVisibility
            originalOrientation = activity.requestedOrientation

            customViewCallback = callback

            decorView.addView(this.view, FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT))
            decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_HIDE_NAVIGATION or View.SYSTEM_UI_FLAG_FULLSCREEN or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION or View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN or View.SYSTEM_UI_FLAG_IMMERSIVE or View.SYSTEM_UI_FLAG_LAYOUT_STABLE
        }

        fun hideViewFromFullScreen() {
            val activity = requireActivity()
            val window = activity.window
            val decorView = window.decorView as FrameLayout

            view?.let {
                decorView.removeView(it)
            }
            decorView.systemUiVisibility = originalSystemUIVisibility
            activity.requestedOrientation = originalOrientation
            customViewCallback?.let {
                it.onCustomViewHidden()
            }
            customViewCallback = null
            view = null
        }
    }
}
