/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.checklist

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.navArgs
import com.commonsense.android.kotlin.system.extensions.onBackPressed
import com.commonsense.android.kotlin.views.extensions.visibleOrGone
import com.google.android.material.snackbar.Snackbar
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.FragmentChecklistDetailsBinding
import dk.nsp.mingraviditet.domain.ExpandCollapseEnum
import dk.nsp.mingraviditet.extensions.measureTextLineCount
import dk.nsp.mingraviditet.util.android.recycler.DelegatingListAdapter
import dk.nsp.mingraviditet.util.extensions.observeIfNotHandled

@AndroidEntryPoint
class ChecklistDetailsFragment : Fragment() {
    private lateinit var binding: FragmentChecklistDetailsBinding

    private val args by navArgs<ChecklistDetailsFragmentArgs>()
    private val viewModel: ChecklistDetailsViewModel by viewModels()

    private val adapter by lazy {
        DelegatingListAdapter(viewLifecycleOwner)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View {
        binding = FragmentChecklistDetailsBinding.inflate(inflater, container, false)
        binding.lifecycleOwner = viewLifecycleOwner
        binding.vm = viewModel

        binding.toolbar.setTitle(R.string.checklist_title)
        binding.toolbar.setNavigationOnClickListener { onBackPressed() }

        binding.recyclerView.adapter = adapter

        binding.recyclerView.itemAnimator = null

        val hasDescription = viewModel.checklistHasDescription()
        binding.expandCollapseButton.visibleOrGone(hasDescription)

        binding.description.addOnLayoutChangeListener { view, _, _, _, _, _, _, _, _ ->
            val textView = view as? TextView ?: return@addOnLayoutChangeListener
            val isMoreThanThreeLines = textView.measureTextLineCount() > 3
            binding.expandCollapseButton.visibleOrGone(hasDescription || isMoreThanThreeLines)
        }

        viewModel.onExpandCollapse.observe(viewLifecycleOwner) { expandCollapseEnum ->
            when (expandCollapseEnum) {
                ExpandCollapseEnum.Expanded -> {
                    binding.description.maxLines = Int.MAX_VALUE
                    binding.expandCollapseButton.setText(R.string.common_cta_collapse)
                }
                ExpandCollapseEnum.Collapsed -> {
                    binding.description.maxLines = 3
                    binding.expandCollapseButton.setText(R.string.common_cta_expand)
                }
                else -> {
                }
            }
        }

        viewModel.checklist.observe(viewLifecycleOwner) {
            binding.progressCircular.setPercentage(it.completeness().toInt())
        }

        viewModel.checklistAdapterViews.observe(viewLifecycleOwner) {
            adapter.submitList(it)
        }

        viewModel.onError.observeIfNotHandled(viewLifecycleOwner) {
            Snackbar.make(binding.root, it, Snackbar.LENGTH_LONG)
                .setAction(R.string.common_ok) {}
                .show()
        }

        return binding.root
    }
}
