/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.knowledge.checklist

import androidx.lifecycle.LiveData
import androidx.lifecycle.MediatorLiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.commonsense.android.kotlin.base.extensions.coroutines.launchIO
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.knowledge.Checklist
import dk.nsp.domain.knowledge.ChecklistItem
import dk.nsp.domain.repository.ChecklistRepository
import dk.nsp.domain.repository.DataCardRepository
import dk.nsp.mingraviditet.domain.ExpandCollapseEnum
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.toSingleEvent
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import javax.inject.Inject

@HiltViewModel
class ChecklistDetailsViewModel @Inject constructor(
    private val checklistRepository: ChecklistRepository,
    private val dataCardRepository: DataCardRepository,
    private val savedStateHandle: SavedStateHandle,
) : ViewModel() {

    private val _onError: MutableLiveData<SingleEvent<String>> =
        MutableLiveData<SingleEvent<String>>()
    val onError: LiveData<SingleEvent<String>> = _onError

    val checklist: LiveData<Checklist> = MediatorLiveData<Checklist>().apply {
        addSource(checklistRepository.checklists) {
            it.firstOrNull { it.id == savedStateHandle.get("checklistId") }
                ?.let { currentChecklist ->
                    postValue(currentChecklist)
                    checklistAdapterViews.postValue(createCards(currentChecklist))
                }
        }
    }

    private val _onExpandCollapse: MutableLiveData<ExpandCollapseEnum> = MutableLiveData()
    val onExpandCollapse: LiveData<ExpandCollapseEnum> = _onExpandCollapse

    fun toggleExpandCollapse() {
        val expanded = _onExpandCollapse.value ?: ExpandCollapseEnum.Collapsed
        _onExpandCollapse.postValue(expanded.toggle())
    }

    val checklistAdapterViews: MutableLiveData<List<RecyclerAdapterView<*>>> = MutableLiveData()

    private val _expandedCards: MutableList<String> = mutableListOf()

    private fun createCards(checklist: Checklist): List<ChecklistCard> {
        return checklist.items.map {
            ChecklistCard(
                ChecklistCardData(
                    it,
                    _expandedCards.contains(it.id),
                    onExpandCollapse = { expanded ->
                        if (expanded == ExpandCollapseEnum.Expanded) {
                            _expandedCards.add(it.id)
                        } else {
                            _expandedCards.remove(it.id)
                        }
                    },
                    onCheck = {
                        toggleChecklistItem(checklist, it)
                    },
                ),
            )
        }
    }

    init {
        dataCardRepository.currentDataCard.observeForever { }
    }

    private fun toggleChecklistItem(checklist: Checklist, checklistItem: ChecklistItem) {
        viewModelScope.launchIO {
            val episodeOfCareId = dataCardRepository.currentDataCard.value?.episodeOfCareIdentifier

            val updatedChecklistItem =
                checklistRepository.updateChecklistItem(
                    this,
                    episodeOfCareId ?: "relates-to-checklist",
                    checklist,
                    checklistItem,
                ).await()

            when (updatedChecklistItem) {
                is Success -> {
                }
                is Failed -> {
                    _onError.postValue(
                        (updatedChecklistItem.error.message ?: "Fejl").toSingleEvent(),
                    )
                    checklistAdapterViews.postValue(createCards(checklist))
                }
            }
        }
    }

    fun checklistHasDescription(): Boolean {
        return checklist.value?.description != null
    }
}
