/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020-2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview

import androidx.lifecycle.LiveData
import androidx.lifecycle.MediatorLiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.map
import androidx.lifecycle.viewModelScope
import com.commonsense.android.kotlin.base.extensions.coroutines.asyncDefault
import com.commonsense.android.kotlin.base.extensions.coroutines.launchDefault
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.articles.Article
import dk.nsp.domain.articles.ArticleCategory
import dk.nsp.domain.articles.RecommendedArticleCategory
import dk.nsp.domain.articles.RecommendedArticleCategoryType
import dk.nsp.domain.repository.ArticlesRepository
import dk.nsp.domain.repository.DataCardRepository
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.extensions.fullName
import dk.nsp.mingraviditet.extensions.getOverviewPregnancyState
import dk.nsp.mingraviditet.extensions.prependIfNotEmpty
import dk.nsp.mingraviditet.extensions.userFullname
import dk.nsp.mingraviditet.ui.adapterviews.FullWidthImageViewRow
import dk.nsp.mingraviditet.ui.adapterviews.ImageViewData
import dk.nsp.mingraviditet.ui.adapterviews.LoadingState
import dk.nsp.mingraviditet.ui.adapterviews.error.ErrorStateRow
import dk.nsp.mingraviditet.ui.common.SectionHeaderAdapterView
import dk.nsp.mingraviditet.ui.main.knowledge.CategoryCard
import dk.nsp.mingraviditet.ui.main.knowledge.CategoryCardData
import dk.nsp.mingraviditet.ui.main.more.documentation.DocumentationCardData
import dk.nsp.mingraviditet.ui.main.more.documentation.DocumentationCardWithButtonRow
import dk.nsp.mingraviditet.ui.main.overview.widget.OverviewPregnancyState
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.postSingleEvent
import dk.nsp.mingraviditet.util.extensions.toSingleEvent
import dk.nsp.mingraviditet.util.extensions.toTextModelOrDefault
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.models.ArticleSearchResult
import dk.nsp.models.DataCard
import dk.nsp.models.DateOfBirth
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.util.extensions.toDefaultMonthDisplay
import dk.nsp.shared.util.extensions.weekNumberAfterBirthDate
import dk.nsp.shared.util.sharedprefs.SharedPrefsHelper
import dk.nsp.shared.valueOrOnFailed
import dk.nsp.util.searchResults
import dk.nsp.util.textmodel.TextModel
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Deferred
import java.time.LocalDateTime
import javax.inject.Inject

@HiltViewModel
class OverviewFragmentViewModel @Inject constructor(
    private val articlesRepository: ArticlesRepository,
    private val dataCardRepository: DataCardRepository,
    private val sharedPrefs: SharedPrefsHelper,
) : ViewModel() {

    private sealed class ArticlesBirthWeekType(val week: Int) {
        abstract val pregnancyType: RecommendedArticleCategoryType

        class PregnantArticles(pregnancyWeek: Int) : ArticlesBirthWeekType(pregnancyWeek) {
            override val pregnancyType: RecommendedArticleCategoryType =
                RecommendedArticleCategoryType.InBirth
        }

        class AfterBirthArticles(weeksAfterBirth: Int) : ArticlesBirthWeekType(weeksAfterBirth) {
            override val pregnancyType: RecommendedArticleCategoryType =
                RecommendedArticleCategoryType.AfterBirth
        }
    }

    val shouldShowChangeDataCardButton = dataCardRepository.hasMultipleDataCards

    private val _rowsToShow = MutableLiveData<List<RecyclerAdapterView<*>>>()
    val rowsToShow: LiveData<List<RecyclerAdapterView<*>>> = _rowsToShow

    private val _onUpdateDateOfBirthClicked = MutableLiveData<SingleEvent<DateOfBirth>>()
    val onUpdateDateOfBirthClicked: LiveData<SingleEvent<DateOfBirth>> = _onUpdateDateOfBirthClicked

    private val _onSearchedArticleClicked: MutableLiveData<SingleEvent<Pair<Article, Int>>> =
        MutableLiveData()
    val onSearchArticleClicked: LiveData<SingleEvent<Pair<Article, Int>>> =
        _onSearchedArticleClicked

    private val _navigateToKnowledge = MutableLiveData<SingleEvent<Unit>>()
    val navigateToKnowledge: LiveData<SingleEvent<Unit>> = _navigateToKnowledge

    private val _navigateToDocumentation = MutableLiveData<SingleEvent<Unit>>()
    val navigateToDocumentation: LiveData<SingleEvent<Unit>> = _navigateToDocumentation

    val usersName = dataCardRepository.currentDataCard.map {
        it?.userFullname
            ?: dataCardRepository.userInfoFromDataCard?.name?.fullName
            ?: ""
    }

    val pregnancyProgress: LiveData<OverviewPregnancyState> =
        MediatorLiveData<OverviewPregnancyState>().apply {
            addSource(dataCardRepository.currentDataCard) {
                postValue(
                    getPregnancyState(
                        it,
                        dataCardRepository.dateOfBirthForCurrentDataCard.value,
                    ),
                )
            }
            addSource(dataCardRepository.dateOfBirthForCurrentDataCard) {
                postValue(
                    getPregnancyState(
                        dataCardRepository.currentDataCard.value,
                        it,
                    ),
                )
            }
        }

    val updateContentListener = pregnancyProgress.map { state ->
        viewModelScope.launchDefault {
            _rowsToShow.postValue(listOf(LoadingState()))

            val pregnancyWeek =
                dataCardRepository.currentDataCard.value?.expectedDateOfDeliverySection?.week

            val weekNumberAfterBirth =
                dataCardRepository.dateOfBirthForCurrentDataCard.value?.weekNumberAfterBirthDate()

            val articlesWeek = articlesBirthWeekTypeFrom(
                weekNumberAfterBirth = weekNumberAfterBirth,
                pregnancyWeek = pregnancyWeek,
            )
            _rowsToShow.postValue(getRows(articlesWeekType = articlesWeek, state = state).await())
        }
    }

    private fun articlesBirthWeekTypeFrom(
        weekNumberAfterBirth: Int?,
        pregnancyWeek: Int?,
    ): ArticlesBirthWeekType {
        return if (weekNumberAfterBirth != null) {
            ArticlesBirthWeekType.AfterBirthArticles(weekNumberAfterBirth)
        } else {
            ArticlesBirthWeekType.PregnantArticles(pregnancyWeek ?: 1)
        }
    }

    private fun onDidGiveBirthPressed() {
        _onUpdateDateOfBirthClicked.postSingleEvent(DateOfBirth())
    }

    private fun presentDateOfBirthCallback(currentDateOfBirth: LocalDateTime?) {
        _onUpdateDateOfBirthClicked.postSingleEvent(DateOfBirth(currentDateOfBirth))
    }

    private fun getPregnancyState(
        currentCard: DataCard?,
        dateOfBirth: LocalDateTime?,
    ): OverviewPregnancyState =
        currentCard.getOverviewPregnancyState(
            dateOfBirth,
            ::onDidGiveBirthPressed,
        ) {
            presentDateOfBirthCallback(dateOfBirth)
        }

    private fun CoroutineScope.getRows(
        articlesWeekType: ArticlesBirthWeekType,
        state: OverviewPregnancyState,
    ): Deferred<List<RecyclerAdapterView<*>>> = asyncDefault {
        when (state) {
            is OverviewPregnancyState.PregnancyEnded -> createEndedRows()
            else -> {
                createArticleRowsForCurrentAndNext(
                    articlesWeekType = articlesWeekType,
                ).await()
            }
        }
    }

    private fun CoroutineScope.createArticleRowsForCurrentAndNext(
        articlesWeekType: ArticlesBirthWeekType,
    ): Deferred<List<RecyclerAdapterView<*>>> = asyncDefault {
        val searchResults = getArticleResultsForWeek(articlesWeekType).await()

        createArticleSection(
            searchResults,
        )
    }

    private fun createEndedRows() = listOf(
        FullWidthImageViewRow(ImageViewData.DrawableFromAttribute(R.attr.sofaIllustration)),
        DocumentationCardWithButtonRow(
            DocumentationCardData(
                TextModel.from(
                    R.string.overview_ended_title,
                    dataCardRepository.currentDataCard.value?.closeDateTime
                        ?.toDefaultMonthDisplay()
                        .toTextModelOrDefault(),
                ),
                TextModel.from(R.string.overview_ended_description),
            ),
        ),
        CategoryCard(
            CategoryCardData.WithoutIcon(
                TextModel.from(R.string.overview_ended_data_title),
                TextModel.from(R.string.overview_ended_data_description),
                TextModel.from(R.string.overview_ended_data_go_to_settings_label),
            ) {
                _navigateToDocumentation.postSingleEvent()
            },
        ),
    )

    private fun createSectionHeader(isFutureSection: Boolean): RecyclerAdapterView<*> {
        val moreButtonText =
            if (!isFutureSection) TextModel.from(R.string.overview_find_more_knowledge_button) else null
        val moreButtonAction = if (!isFutureSection) {
            { _navigateToKnowledge.postSingleEvent() }
        } else {
            null
        }

        return SectionHeaderAdapterView(
            TextModel.from(
                if (isFutureSection) R.string.overview_relevant_next_week_title else R.string.overview_relevant_right_now_title,
            ),
            moreButtonText,
            moreButtonAction,
        )
    }

    private fun createArticleSection(
        result: ResultWrapper<RecommendedArticleCategory>,
    ): List<RecyclerAdapterView<*>> {
        val value = result.valueOrOnFailed {
            return listOf(
                createSectionHeader(isFutureSection = false),
                ErrorStateRow(
                    title = TextModel.from(R.string.common_alert_title_error),
                    description = TextModel.from("${it.message}"),
                ),
            )
        }
        return value.createCurrentWeekCards() + value.createNextWeekCards()
    }

    private fun RecommendedArticleCategory.createCurrentWeekCards(): List<RecyclerAdapterView<*>> {
        return createCardsForWeek(week = currentWeek, isNextWeek = false)
    }

    private fun RecommendedArticleCategory.createNextWeekCards(): List<RecyclerAdapterView<*>> {
        return createCardsForWeek(week = nextWeek, isNextWeek = true)
    }

    private fun createCardsForWeek(
        week: List<ArticleCategory>,
        isNextWeek: Boolean,
    ): List<RecyclerAdapterView<*>> {
        return week
            .mapToCardForSearchResults()
            .prependIfNotEmpty {
                createSectionHeader(isFutureSection = isNextWeek)
            }
    }

    private fun List<ArticleCategory>.mapToCardForSearchResults(): List<CategoryCard> = flatMap { it.searchResults() }.map {
        createCardForSearchResult(it)
    }

    private fun CoroutineScope.getArticleResultsForWeek(
        articlesWeekType: ArticlesBirthWeekType,
    ): Deferred<ResultWrapper<RecommendedArticleCategory>> = asyncDefault {
        with(articlesWeekType.pregnancyType) {
            getRecommendedArticles(
                week = articlesWeekType.week,
                repository = articlesRepository,
            )
        }.await()
    }

    private fun createCardForSearchResult(
        articleSearchResult: ArticleSearchResult,
    ): CategoryCard = CategoryCard(
        CategoryCardData.WithIconUrl(
            mTitle = articleSearchResult.title,
            mDescription = articleSearchResult.description,
            mTickerText = TextModel.from(articleSearchResult.subtitle),
            iconUrl = articleSearchResult.iconUrl,
        ) {
            _onSearchedArticleClicked.postValue(
                Pair(
                    articleSearchResult.article,
                    articleSearchResult.articleSectionId ?: -1,
                ).toSingleEvent(),
            )
        },
    )
}
