/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.dateofbirth

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.FragmentUpdateDateOfBirthBinding
import dk.nsp.mingraviditet.domain.AlertDialogButtonModel
import dk.nsp.mingraviditet.domain.AlertDialogViewModel
import dk.nsp.mingraviditet.extensions.disable
import dk.nsp.mingraviditet.extensions.enable
import dk.nsp.mingraviditet.extensions.getDetails
import dk.nsp.mingraviditet.extensions.visibleOnTrue
import dk.nsp.mingraviditet.util.extensions.createAndShowWith
import dk.nsp.mingraviditet.util.extensions.observeIfNotHandled
import dk.nsp.models.DateOfBirth
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import java.time.LocalDateTime

@AndroidEntryPoint
class UpdateDateOfBirthFragment : Fragment() {

    private val vm: UpdateDateOfBirthViewModel by viewModels()

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View {
        val binding = FragmentUpdateDateOfBirthBinding.inflate(inflater, container, false)
        binding.lifecycleOwner = viewLifecycleOwner
        binding.confirmButton.setOnClickListener {
            binding.confirmButton.disable()

            vm.sendUpdateDateOfBirthToBackend()
        }

        binding.resetDate.setOnClickListener {
            showConfirmResetDateOfBirthDialog(binding.root.context)
        }

        setupCalendarView(binding)

        vm.hasInitialDateOfBirth.observeIfNotHandled(viewLifecycleOwner) { hasInitialDateOfBirth ->
            binding.resetDate.visibleOnTrue(hasInitialDateOfBirth)
        }

        vm.onDateOfBirthUpdated.observe(viewLifecycleOwner) {
            when (it) {
                is Success -> {
                    findNavController().popBackStack()
                }
                is Failed -> {
                    binding.confirmButton.enable()
                    val alertDialogViewModel = AlertDialogViewModel(
                        R.string.common_alert_title_error,
                        R.string.common_alert_message_generic,
                        positiveButton = AlertDialogButtonModel(R.string.common_ok),
                        details = it.error.getDetails(),
                    )
                    context?.let { ctx ->
                        AlertDialog
                            .Builder(ctx)
                            .createAndShowWith(alertDialogViewModel)
                    }
                }
            }
        }

        return binding.root
    }

    private fun setupCalendarView(binding: FragmentUpdateDateOfBirthBinding) {
        binding.calendar.maxDate =
            System.currentTimeMillis() // Don't allow for selection in the future

        binding.calendar.setOnDateChangedListener { _, year, monthOfYear, dayOfMonth ->
            vm.updateLocalDateOfBirth(
                DateOfBirth(
                    LocalDateTime.of(
                        year,
                        monthOfYear + 1,
                        dayOfMonth,
                        12,
                        0,
                    ),
                ),
            )
        }

        vm.pickedDateOfBirth.observe(viewLifecycleOwner) {
            it.dateOfBirth?.let {
                if (it.year != binding.calendar.year || it.monthValue != binding.calendar.month - 1 || it.dayOfMonth != binding.calendar.dayOfMonth) {
                    binding.calendar.updateDate(it.year, it.monthValue - 1, it.dayOfMonth)
                }
            }
        }
    }

    private fun showConfirmResetDateOfBirthDialog(context: Context) {
        AlertDialog.Builder(context).createAndShowWith(
            AlertDialogViewModel(
                R.string.overview_reset_date_of_birth_alert_title,
                R.string.overview_reset_date_of_birth_alert_description,
                positiveButton = AlertDialogButtonModel(R.string.common_cta_reset) { _, _ ->
                    vm.clearDateOfBirthToBackend()
                },
                negativeButton = AlertDialogButtonModel(R.string.common_regret) { _, _ ->
                },
            ),
        )
    }
    companion object {
        const val dateArgKey = "dateOfBirthArgKey"
    }
}
