/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.dateofbirth

import androidx.lifecycle.LiveData
import androidx.lifecycle.MediatorLiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.map
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.repository.DataCardRepository
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.models.DateOfBirth
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.util.launchDefault
import java.time.LocalDateTime
import javax.inject.Inject

@HiltViewModel
class UpdateDateOfBirthViewModel @Inject constructor(
    private val dataCardRepository: DataCardRepository,
    private val savedStateHandle: SavedStateHandle,
) : ViewModel() {

    private val _initialDateOfBirth = MutableLiveData<LocalDateTime?>(
        savedStateHandle.get<DateOfBirth?>(
            savedStateHandle.get<String>(UpdateDateOfBirthFragment.dateArgKey) ?: "",
        )?.dateOfBirth,
    )

    val hasInitialDateOfBirth =
        _initialDateOfBirth.map { initialDateOfBirth -> SingleEvent(initialDateOfBirth != null) }

    private val _pickedDateOfBirth = MediatorLiveData<DateOfBirth>().apply {
        addSource(_initialDateOfBirth) {
            postValue(DateOfBirth(it ?: LocalDateTime.now()))
        }
    }
    val pickedDateOfBirth: LiveData<DateOfBirth> = _pickedDateOfBirth

    private val _onDateOfBirthUpdated = MutableLiveData<ResultWrapper<DateOfBirth>>()
    val onDateOfBirthUpdated: LiveData<ResultWrapper<DateOfBirth>> = _onDateOfBirthUpdated

    fun updateLocalDateOfBirth(dateOfBirth: DateOfBirth) {
        if (dateOfBirth != _pickedDateOfBirth.value) {
            _pickedDateOfBirth.postValue(dateOfBirth)
        }
    }

    fun sendUpdateDateOfBirthToBackend() {
        viewModelScope.launchDefault {
            val updateRes =
                dataCardRepository.updateDateOfBirthForCurrentSelection(
                    this,
                    _pickedDateOfBirth.value?.dateOfBirth,
                ).await()

            _onDateOfBirthUpdated.postValue(updateRes)
        }
    }

    fun clearDateOfBirthToBackend() {
        viewModelScope.launchDefault {
            val updateRes =
                dataCardRepository.updateDateOfBirthForCurrentSelection(this, null).await()

            _onDateOfBirthUpdated.postValue(updateRes)
        }
    }
}
