/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.useradmission.login

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.commonsense.android.kotlin.base.extensions.coroutines.launchIO
import dagger.hilt.android.lifecycle.HiltViewModel
import dk.nsp.domain.repository.ConsentRepository
import dk.nsp.domain.repository.DataCardRepository
import dk.nsp.domain.repository.DictionaryRepository
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.domain.AlertDialogButtonModel
import dk.nsp.mingraviditet.domain.AlertDialogViewModel
import dk.nsp.mingraviditet.extensions.getDetails
import dk.nsp.mingraviditet.util.extensions.getValue
import dk.nsp.mingraviditet.util.extensions.postSingleEvent
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import javax.inject.Inject

@HiltViewModel
class InitialDataFetchViewModel @Inject constructor(
    private val dataCardRepository: DataCardRepository,
    private val dictionaryRepository: DictionaryRepository,
    val consentRepository: ConsentRepository,
) : ViewModel() {

    private val _successFullyLoadedData = MutableLiveData<SingleEvent<Boolean>>()
    val successFullyLoadedData by _successFullyLoadedData

    private val _triggerDataFetchFailedDialog = MutableLiveData<SingleEvent<AlertDialogViewModel>>()
    val triggerDataFetchFailedDialog by _triggerDataFetchFailedDialog

    fun tryInitializeUserData() {
        viewModelScope.launchIO {
            dictionaryRepository.getDictionary(this)

            when (val result = dataCardRepository.getDataCards(this).await()) {
                is Failed -> {
                    _successFullyLoadedData.postSingleEvent(false)
                    _triggerDataFetchFailedDialog.postSingleEvent(
                        AlertDialogViewModel(
                            R.string.login_loading_data_failed_alert_title,
                            R.string.common_alert_message_generic,
                            positiveButton = AlertDialogButtonModel(R.string.common_ok),
                            details = result.error.getDetails(),
                        ),
                    )
                }
                is Success -> _successFullyLoadedData.postSingleEvent(true)
            }
        }
    }
}
