/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.extensions

import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.testData.CprIdentifiedPersonEmpty
import dk.nsp.mingraviditet.testData.DataCardEmpty
import dk.nsp.mingraviditet.testData.ExpectedDateOfDeliverySectionEmpty
import dk.nsp.mingraviditet.testData.MultiplePregnancySectionEmpty
import dk.nsp.mingraviditet.testextensions.assertResourceModel
import dk.nsp.mingraviditet.testextensions.assertStringModel
import dk.nsp.mingraviditet.ui.main.overview.widget.OverviewPregnancyState
import dk.nsp.models.DataCard
import dk.nsp.models.PersonName
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIsAndApply
import dk.nsp.testassertions.assertNull
import dk.nsp.util.textmodel.TextResourcePlaceholderModel
import org.junit.jupiter.api.Test
import java.time.LocalDateTime

class DataCardExtensionsTest {
    @Test
    fun dataCardUserFullname() {
        DataCardEmpty.copy(
            recordTarget = CprIdentifiedPersonEmpty.copy(
                PersonName(
                    firstNames = listOf("Søren Peder"),
                    middleNames = listOf("Ole", "Piet"),
                    lastName = "Hansen",
                ),
            ),
        ).userFullname.assertEquals("Søren Peder Ole Piet Hansen")

        DataCardEmpty.copy(
            recordTarget = CprIdentifiedPersonEmpty.copy(
                name = null,
            ),
        ).userFullname.assertNull()
    }

    @Test
    fun dataCardTrimester() {
        DataCardEmpty.copy(
            expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                trimester = 2,
            ),
        ).trimester.assertEquals(2)

        DataCardEmpty.copy(
            expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                trimester = null,
            ),
        ).trimester.assertNull()

        DataCardEmpty.copy(
            expectedDateOfDeliverySection = null,
        ).trimester.assertNull()
    }

    @Test
    fun dataCardWeek() {
        DataCardEmpty.copy(
            expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                week = 32,
            ),
        ).week.assertEquals(32)

        DataCardEmpty.copy(
            expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                week = null,
            ),
        ).week.assertNull()

        DataCardEmpty.copy(
            expectedDateOfDeliverySection = null,
        ).week.assertNull()
    }

    @Test
    fun dataCardDueDate() {
        val dueDate = LocalDateTime.of(2021, 11, 12, 0, 0)
        DataCardEmpty.copy(
            expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                dateOfDelivery = dueDate,
            ),
        ).dueDate.assertEquals(dueDate)

        DataCardEmpty.copy(
            expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                dateOfDelivery = null,
            ),
        ).dueDate.assertNull()

        DataCardEmpty.copy(
            expectedDateOfDeliverySection = null,
        ).dueDate.assertNull()
    }

    @Test
    fun dataCardGestationAge() {
        DataCardEmpty.copy(
            expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                gestationAge = "32",
            ),
        ).gestationAge.assertEquals("32")

        DataCardEmpty.copy(
            expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                gestationAge = null,
            ),
        ).gestationAge.assertNull()

        DataCardEmpty.copy(
            expectedDateOfDeliverySection = null,
        ).gestationAge.assertNull()
    }

    @Test
    fun dataCardNumberOfFetuses() {
        DataCardEmpty.copy(
            multiplePregnancySection = MultiplePregnancySectionEmpty.copy(
                numberOfFetuses = 2,
            ),
        ).numberOfFetuses.assertEquals(2)

        DataCardEmpty.copy(
            multiplePregnancySection = MultiplePregnancySectionEmpty.copy(
                numberOfFetuses = null,
            ),
        ).numberOfFetuses.assertEquals(1)

        DataCardEmpty.copy(
            multiplePregnancySection = null,
        ).numberOfFetuses.assertEquals(1)

        val nullable: DataCard? = null
        nullable.numberOfFetuses.assertEquals(1)
    }

    class DataCardChangeDataCardTitle {
        @Test
        fun endedWithCloseDateTime() {
            DataCardEmpty.copy(
                documentStatus = DataCard.DocumentStatus.CLOSED,
                closeDateTime = LocalDateTime.of(2021, 9, 8, 3, 2),
                expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                    dateOfDelivery = null,
                ),
            ).changeDataCardTitle.assertIsAndApply<TextResourcePlaceholderModel> {
                textRes.assertEquals(R.string.overview_pick_pregnancy_ended_date)

                placeholders.size.assertEquals(1)
                placeholders.firstOrNull().assertStringModel("08.09.2021")
            }
        }

        @Test
        fun notEndedAndNoDueDate() {
            DataCardEmpty.copy(
                documentStatus = DataCard.DocumentStatus.ACTIVE,
                closeDateTime = null,
                expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                    dateOfDelivery = LocalDateTime.of(2021, 2, 4, 20, 4),
                ),
            ).changeDataCardTitle.assertIsAndApply<TextResourcePlaceholderModel> {
                textRes.assertEquals(R.string.overview_pick_pregnancy_active_date)

                placeholders.size.assertEquals(1)
                placeholders.firstOrNull().assertStringModel("04.02.2021")
            }
        }

        @Test
        fun fallthroughCase() {
            DataCardEmpty.copy(
                documentStatus = null,
                closeDateTime = null,
                expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                    dateOfDelivery = null,
                ),
            ).changeDataCardTitle.assertResourceModel(R.string.common_missing_data)

            DataCardEmpty.copy(
                documentStatus = DataCard.DocumentStatus.CLOSED,
                closeDateTime = null,
                expectedDateOfDeliverySection = null,
            ).changeDataCardTitle.assertResourceModel(R.string.common_missing_data)
        }
    }

    class DataCardGetOverviewPregnancyState {
        @Test
        fun stateNone() {
            DataCardEmpty.copy(
                expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                    dateOfDelivery = null,
                ),
            ).getOverviewPregnancyState(null, {}, {})
                .assertEquals(OverviewPregnancyState.None)

            val nullable: DataCard? = null
            nullable.getOverviewPregnancyState(null, {}, {})
                .assertEquals(OverviewPregnancyState.None)
        }

        @Test
        fun statePregnancyEnded() {
            DataCardEmpty.copy(
                expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                    dateOfDelivery = LocalDateTime.of(2021, 4, 5, 13, 4),
                ),
                documentStatus = DataCard.DocumentStatus.CLOSED,
                multiplePregnancySection = MultiplePregnancySectionEmpty.copy(
                    numberOfFetuses = 2,
                ),
            ).getOverviewPregnancyState(null, {}, {})
                .assertIsAndApply<OverviewPregnancyState.PregnancyEnded> {
                    numberOfFetuses.assertEquals(2)
                }
        }

        @Test
        fun stateActiveAndGaveBirth() {
            val dateOfBirth = LocalDateTime.of(2021, 4, 7, 13, 4)

            DataCardEmpty.copy(
                expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                    dateOfDelivery = LocalDateTime.of(2021, 4, 5, 13, 4),
                ),
                multiplePregnancySection = MultiplePregnancySectionEmpty.copy(
                    numberOfFetuses = 1,
                ),
            ).getOverviewPregnancyState(dateOfBirth, {}, {})
                .assertIsAndApply<OverviewPregnancyState.ActiveAndGaveBirth> {
                    this.dateOfBirth.assertEquals(dateOfBirth)
                    this.numberOfFetuses.assertEquals(1)
                }
        }

        @Test
        fun stateActiveAndWaitingForBirth() {
            val dueDate = LocalDateTime.of(2021, 4, 5, 13, 4)

            val withGestationAge = DataCardEmpty.copy(
                expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                    dateOfDelivery = dueDate,
                    trimester = 3,
                    gestationAge = "34",
                    week = 35,
                    monthsIntoPregnancy = 7.3,
                ),
                multiplePregnancySection = MultiplePregnancySectionEmpty.copy(
                    numberOfFetuses = 1,
                ),
            )
            withGestationAge.getOverviewPregnancyState(null, {}, {})
                .assertIsAndApply<OverviewPregnancyState.ActiveAndWaitingForBirth> {
                    this.dueDate.assertEquals(dueDate)
                    this.gestationAgeString.assertEquals("34")
                    this.monthsIntoPregnancy.assertEquals(7.3)
                    this.numberOfFetuses.assertEquals(1)
                    this.trimester.assertEquals(3)
                    this.week.assertEquals(35)
                }

            val withoutGestationAge = DataCardEmpty.copy(
                expectedDateOfDeliverySection = ExpectedDateOfDeliverySectionEmpty.copy(
                    dateOfDelivery = dueDate,
                    trimester = 3,
                    gestationAge = null,
                    week = 35,
                    monthsIntoPregnancy = 7.3,
                ),
                multiplePregnancySection = MultiplePregnancySectionEmpty.copy(
                    numberOfFetuses = 1,
                ),
            )
            withoutGestationAge.getOverviewPregnancyState(null, {}, {})
                .assertIsAndApply<OverviewPregnancyState.ActiveAndWaitingForBirth> {
                    this.dueDate.assertEquals(dueDate)
                    this.gestationAgeString.assertNull()
                    this.monthsIntoPregnancy.assertEquals(7.3)
                    this.numberOfFetuses.assertEquals(1)
                    this.trimester.assertEquals(3)
                    this.week.assertEquals(35)
                }
        }
    }
}
