/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021-2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.appointments.extensions

import android.os.Build
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.util.extensions.getColorFromAttr
import dk.nsp.models.*
import dk.nsp.shared.util.extensions.withTimeAtStartOfDay
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertFalse
import dk.nsp.testassertions.assertIsAndApply
import dk.nsp.testassertions.assertTrue
import dk.nsp.util.textmodel.TextResourceModel
import dk.nsp.util.textmodel.TextStringModel
import org.junit.jupiter.api.Test
import org.junit.runner.RunWith
import org.robolectric.RobolectricTestRunner
import org.robolectric.RuntimeEnvironment
import org.robolectric.annotation.Config
import java.time.LocalDateTime

private val context
    get() = RuntimeEnvironment.getApplication()

@Config(sdk = [Build.VERSION_CODES.P], manifest = Config.NONE)
@RunWith(RobolectricTestRunner::class)
class ActivityExtensionsTest {

    private val observation =
        Observation(
            LocalDateTime.now().minusWeeks(1),
            listOf(ObservationValues("65", "kg", "Vægt")),
            measuredBy = HealthCarePerformer(
                person = Person(
                    PersonName(
                        prefixName = "Dr.",
                        firstNames = listOf("Peter"),
                        middleNames = listOf("MiddleName"),
                        lastName = "Hansen",
                    ),
                ),
                role = "Sygeplejerske",
                healthCareOrganization = null,
                healthCareOrganizationAdress = null,
                healthCareOrganizationPhoneNumber = null,
            ),
            "Fetus 1",
            "actRef",
            ObservationType.Weight,
            "measurementId",
        )

    private val note = Note(
        "Note title",
        "Content",
        LocalDateTime.now(),
        null,
        null,
        null,
        "DocumentUniqueId",
        false,
    )

    private val minimalActivity = Activity(
        Activity.Status.Expected,
        "minimalCareplanActivity",
        null,
        null,
        null,
        null,
        null,
        null,
        null,
        null,
        null,
        null,
        null,
        null,
        null,
    )

    private val filledActivity = Activity(
        Activity.Status.Expected,
        "filledCareplanActivity",
        ActivityPractitioner("FirstName", "Test", "Testesen", "Test Org", "12345678", null, null),
        ActivityLocation(
            null,
            ActivityLocationAddress("Hedeager 5", "8200", "Aarhus N"),
            "Hospitalet",
            "12345678",
            "1@email.com",
            null,
        ),
        "11+2",
        ActivityPlannedTime(
            LocalDateTime.now().withTimeAtStartOfDay().plusDays(2),
            LocalDateTime.now().withTimeAtStartOfDay().plusDays(2).plusHours(1),
        ),
        null,
        Activity.Ownership.CurrentUser,
        "Description",
        "Reference",
        Code("Code", "CodeSystem", "Display"),
        "Kind",
        "SomeId",
        "3213",
        note,
        listOf(observation),
        LocalDateTime.now(),
    )

    @Test
    fun hasObservationsOrNotes() {
        val emptyNote: Note? = null
        val emptyObservations = listOf<Observation>()

        val careplanWithEmptyObservationsAndNotes =
            minimalActivity.copy(note = emptyNote, observations = emptyObservations)

        careplanWithEmptyObservationsAndNotes.hasObservationsOrNotes().assertFalse()
        filledActivity.copy(observations = listOf(observation)).hasObservationsOrNotes()
            .assertTrue()
    }

    @Test
    fun activityIsPlannedOrExpected() {
        minimalActivity.copy(status = Activity.Status.Expected).isPlannedOrExpected().assertTrue()
        minimalActivity.copy(status = Activity.Status.Planned).isPlannedOrExpected().assertTrue()
        minimalActivity.copy(status = Activity.Status.Finished).isPlannedOrExpected().assertFalse()
        minimalActivity.copy(status = Activity.Status.Cancelled).isPlannedOrExpected().assertFalse()
    }

    @Test
    fun activityIsFinished() {
        minimalActivity.copy(status = Activity.Status.Finished).isFinished().assertTrue()
        minimalActivity.copy(status = Activity.Status.Expected).isFinished().assertFalse()
        minimalActivity.copy(status = Activity.Status.Planned).isFinished().assertFalse()
        minimalActivity.copy(status = Activity.Status.Cancelled).isFinished().assertFalse()
    }

    @org.junit.Test
    fun activityGetSubtitleText() {
        // With planned time
        val withPlannedTime = minimalActivity.copy(
            plannedTime = ActivityPlannedTime(
                from = LocalDateTime.of(2021, 4, 16, 12, 59, 1, 99),
            ),
        )
        withPlannedTime.getSubtitleText().assertIsAndApply<TextStringModel> {
            text.assertEquals("Apr 16, 2021")
        }

        // With gestationWeek
        val withGestationWeek = minimalActivity.copy(scheduledString = "gestWeek")
        withGestationWeek.getSubtitleText().assertIsAndApply<TextStringModel> {
            text.assertEquals("gestWeek")
        }

        // With null
        minimalActivity.getSubtitleText().assertIsAndApply<TextResourceModel> {
            textRes.assertEquals(R.string.common_missing_data)
        }
    }

    @org.junit.Test
    fun activityGetStatusChipBackgroundColor() {
        minimalActivity.copy(status = Activity.Status.Finished)
            .getStatusChipBackgroundColor(context).assertEquals(
                context.getColorFromAttr(R.attr.activityStatusFinishedBackgroundColor),
            )

        minimalActivity.copy(status = Activity.Status.Planned).getStatusChipBackgroundColor(context)
            .assertEquals(
                context.getColorFromAttr(R.attr.activityStatusPlannedBackgroundColor),
            )

        val nullActivity: Activity? = null
        nullActivity.getStatusChipBackgroundColor(context).assertEquals(
            context.getColorFromAttr(R.attr.activityStatusPlannedBackgroundColor),
        )
    }

    @Test
    fun activityGetStatusTextModel() {
        minimalActivity.copy(status = Activity.Status.Finished).getStatusTextModel()
            .assertIsAndApply<TextResourceModel> {
                textRes.assertEquals(R.string.appointment_details_finished_badge)
            }
        minimalActivity.copy(status = Activity.Status.Planned).getStatusTextModel()
            .assertIsAndApply<TextResourceModel> {
                textRes.assertEquals(R.string.appointment_details_planned_badge)
            }
    }
}
