/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.journal.extensions

import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.testextensions.assertResourceModel
import dk.nsp.mingraviditet.testextensions.assertStringModel
import dk.nsp.models.AllergiesSection
import dk.nsp.models.AssistedFertilizationSection
import dk.nsp.models.ChronicDiseasesSection
import dk.nsp.models.ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod
import dk.nsp.models.MidwifeClinicSectionAllOfParentalCraftClassDesired
import dk.nsp.models.MidwifeConsultationSectionAllOfMidwifeConsultationWishes
import dk.nsp.models.MmrVaccinationSection
import dk.nsp.models.OverallAssessmentSectionAllOfPrenatalCareAssessment
import dk.nsp.models.PosNegEnum
import dk.nsp.models.PositiveNegativeDeclinedScreeningWithDateSection
import dk.nsp.models.PreviousAbortionsSectionAllOfAbortions
import dk.nsp.models.PreviousBirthsSectionAllOfBirths
import dk.nsp.models.PreviousBirthsSectionAllOfChildren
import dk.nsp.models.RiskAssessment3
import dk.nsp.models.SocialHistorySection
import dk.nsp.models.TobaccoUseAndExposureSection
import dk.nsp.models.WorkEnvironmentImpactSectionAllOfMother
import dk.nsp.models.WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk
import dk.nsp.models.YesNoEnum
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIsAndApply
import dk.nsp.util.textmodel.TextCompositeModel
import org.junit.jupiter.api.Test

class EnumsToTextModelTest {

    @Test
    fun yesNoEnumToTextModel() {
        YesNoEnum.True.toTextModel().assertResourceModel(expected = R.string.enum_yes)
        YesNoEnum.False.toTextModel().assertResourceModel(expected = R.string.enum_no)

        val nullable: YesNoEnum? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun posNegEnumToTextModel() {
        PosNegEnum.Positive.toTextModel().assertResourceModel(expected = R.string.enum_positive)
        PosNegEnum.Negative.toTextModel().assertResourceModel(expected = R.string.enum_negative)

        val nullable: PosNegEnum? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun positiveNegativeDeclinedScreeningWithDateSectionResultToTextModel() {
        PositiveNegativeDeclinedScreeningWithDateSection.Result.Positive.toTextModel()
            .assertResourceModel(expected = R.string.enum_positive)
        PositiveNegativeDeclinedScreeningWithDateSection.Result.Negative.toTextModel()
            .assertResourceModel(expected = R.string.enum_negative)
        PositiveNegativeDeclinedScreeningWithDateSection.Result.Declined.toTextModel()
            .assertResourceModel(expected = R.string.enum_declined)

        val nullable: PositiveNegativeDeclinedScreeningWithDateSection.Result? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun expectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriodCertaintyToTextModel() {
        ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod.Certainty.Certain.toTextModel()
            .assertResourceModel(expected = R.string.enum_certainty_certain)
        ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod.Certainty.Equivocal.toTextModel()
            .assertResourceModel(expected = R.string.enum_certainty_equivocal)

        val nullable: ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod.Certainty? =
            null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun AssistedFertilizationSectionTypeToTextModel() {
        AssistedFertilizationSection.Type.Spontaneous.toTextModel()
            .assertResourceModel(expected = R.string.assisted_fertilization_type_spontaneous)
        AssistedFertilizationSection.Type.OvulationInductionIvf.toTextModel()
            .assertResourceModel(expected = R.string.assisted_fertilization_type_ovulation_induction_ivf)
        AssistedFertilizationSection.Type.Ivf.toTextModel()
            .assertResourceModel(expected = R.string.assisted_fertilization_type_ivf)
        AssistedFertilizationSection.Type.Icsi.toTextModel()
            .assertResourceModel(expected = R.string.assisted_fertilization_type_icsi)
        AssistedFertilizationSection.Type.HormoneTherapy.toTextModel()
            .assertResourceModel(expected = R.string.assisted_fertilization_type_hormone_therapy)
        AssistedFertilizationSection.Type.Insemination.toTextModel()
            .assertResourceModel(expected = R.string.assisted_fertilization_type_insemination)
        AssistedFertilizationSection.Type.EggDonation.toTextModel()
            .assertResourceModel(expected = R.string.assisted_fertilization_type_egg_donation)
        AssistedFertilizationSection.Type.Unknown.toTextModel()
            .assertResourceModel(expected = R.string.assisted_fertilization_type_unknown)

        val nullable: AssistedFertilizationSection.Type? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun allergiesSectionAllergicPredispositionOfChildToTextModel() {
        AllergiesSection.AllergicPredispositionOfChild.NoPredisposition.toTextModel()
            .assertResourceModel(expected = R.string.allergic_predisposition_no)
        AllergiesSection.AllergicPredispositionOfChild.SinglePredisposition.toTextModel()
            .assertResourceModel(expected = R.string.allergic_predisposition_single)
        AllergiesSection.AllergicPredispositionOfChild.DoublePredisposition.toTextModel()
            .assertResourceModel(expected = R.string.allergic_predisposition_double)
        val nullable: AllergiesSection.AllergicPredispositionOfChild? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun chronicDiseasesSectionConditionsToStringRes() {
        ChronicDiseasesSection.Conditions.CirculatorySystem.toStringRes()
            .assertEquals(R.string.chronic_condition_circulatory_system)
        ChronicDiseasesSection.Conditions.RespiratorySystem.toStringRes()
            .assertEquals(R.string.chronic_condition_respiratory_system)
        ChronicDiseasesSection.Conditions.Thyroid.toStringRes()
            .assertEquals(R.string.chronic_condition_thyroid)
        ChronicDiseasesSection.Conditions.Diabetes.toStringRes()
            .assertEquals(R.string.chronic_condition_diabetes)
        ChronicDiseasesSection.Conditions.Epilepsy.toStringRes()
            .assertEquals(R.string.chronic_condition_epilepsy)
        ChronicDiseasesSection.Conditions.MentalIllness.toStringRes()
            .assertEquals(R.string.chronic_condition_mental_illness)
        ChronicDiseasesSection.Conditions.Herpes.toStringRes()
            .assertEquals(R.string.chronic_condition_herpes)
        ChronicDiseasesSection.Conditions.Uvi.toStringRes()
            .assertEquals(R.string.chronic_condition_uvi)
        ChronicDiseasesSection.Conditions.ConeSection.toStringRes()
            .assertEquals(R.string.chronic_condition_cone_section)
        ChronicDiseasesSection.Conditions.Mrsa.toStringRes()
            .assertEquals(R.string.chronic_condition_mrsa)
        ChronicDiseasesSection.Conditions.Gbs.toStringRes()
            .assertEquals(R.string.chronic_condition_gbs)
    }

    class ListChronicDiseasesSectionConditionsToTextModel {
        @Test
        fun nullable() {
            emptyList<ChronicDiseasesSection.Conditions>().toTextModel()
                .assertResourceModel(expected = R.string.common_missing_data)
        }

        @Test
        fun empty() {
            emptyList<ChronicDiseasesSection.Conditions>().toTextModel()
                .assertResourceModel(expected = R.string.common_missing_data)
        }

        @Test
        fun single() {
            val result = listOf(ChronicDiseasesSection.Conditions.Thyroid).toTextModel()

            result.assertIsAndApply<TextCompositeModel> {
                val models = bindings
                models.size.assertEquals(1)
                val thyroid = models.firstOrNull()
                thyroid.assertResourceModel(expected = R.string.chronic_condition_thyroid)

                seperator.assertStringModel(expected = ", ")
            }
        }

        @Test
        fun multiple() {
            val result = listOf(
                ChronicDiseasesSection.Conditions.MentalIllness,
                ChronicDiseasesSection.Conditions.Diabetes,
                ChronicDiseasesSection.Conditions.ConeSection,
            ).toTextModel()

            result.assertIsAndApply<TextCompositeModel> {
                val models = bindings
                models.size.assertEquals(3)

                val mentalIllness = models.getOrNull(0)
                val diabetes = models.getOrNull(1)
                val coneSection = models.getOrNull(2)

                mentalIllness.assertResourceModel(expected = R.string.chronic_condition_mental_illness)
                diabetes.assertResourceModel(expected = R.string.chronic_condition_diabetes)
                coneSection.assertResourceModel(expected = R.string.chronic_condition_cone_section)

                seperator.assertStringModel(expected = ", ")
            }
        }
    }

    @Test
    fun mmrVaccinationSectionStatusToTextModel() {
        MmrVaccinationSection.Status.Vaccinated.toTextModel()
            .assertResourceModel(expected = R.string.vaccination_status_vaccinated)
        MmrVaccinationSection.Status.NotVaccinated.toTextModel()
            .assertResourceModel(expected = R.string.vaccination_status_not_vaccinated)
        MmrVaccinationSection.Status.Unknown.toTextModel()
            .assertResourceModel(expected = R.string.vaccination_status_unknown)

        val nullable: MmrVaccinationSection.Status? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun tobaccoUseAndExposureSectionTobaccoUserToTextModel() {
        TobaccoUseAndExposureSection.TobaccoUser.Active.toTextModel()
            .assertResourceModel(expected = R.string.tobacco_usage_active)
        TobaccoUseAndExposureSection.TobaccoUser.Never.toTextModel()
            .assertResourceModel(expected = R.string.tobacco_usage_never)
        TobaccoUseAndExposureSection.TobaccoUser.FormerBecausePregnancy.toTextModel()
            .assertResourceModel(expected = R.string.tobacco_usage_former_because_pregnancy)
        TobaccoUseAndExposureSection.TobaccoUser.FormerLongAgo.toTextModel()
            .assertResourceModel(expected = R.string.tobacco_usage_former_long_ago)
        val nullable: TobaccoUseAndExposureSection.TobaccoUser? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun workEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRiskTypesToStringRes() {
        WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types.Biological.toStringRes()
            .assertEquals(R.string.work_environment_risk_type_biological)
        WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types.Chemical.toStringRes()
            .assertEquals(R.string.work_environment_risk_type_chemical)
        WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types.Ergonomic.toStringRes()
            .assertEquals(R.string.work_environment_risk_type_ergonomic)
        WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types.Other.toStringRes()
            .assertEquals(R.string.work_environment_risk_type_other)
    }

    @Test
    fun workEnvironmentImpactSectionAllOfMotherPrenatalSickLeaveToTextModel() {
        WorkEnvironmentImpactSectionAllOfMother.PrenatalSickLeave.None.toTextModel()
            .assertResourceModel(expected = R.string.prenatal_sick_leave_none)
        WorkEnvironmentImpactSectionAllOfMother.PrenatalSickLeave.Partial.toTextModel()
            .assertResourceModel(expected = R.string.prenatal_sick_leave_partial)
        WorkEnvironmentImpactSectionAllOfMother.PrenatalSickLeave.Full.toTextModel()
            .assertResourceModel(expected = R.string.prenatal_sick_leave_full)

        val nullable: WorkEnvironmentImpactSectionAllOfMother.PrenatalSickLeave? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun previousBirthsSectionAllOfChildrenSexToTextModel() {
        PreviousBirthsSectionAllOfChildren.Sex.Female.toTextModel()
            .assertResourceModel(expected = R.string.child_sex_female)
        PreviousBirthsSectionAllOfChildren.Sex.Male.toTextModel()
            .assertResourceModel(expected = R.string.child_sex_male)

        val nullable: PreviousBirthsSectionAllOfChildren.Sex? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun previousBirthsSectionAllOfChildrenOutcomeToTextModel() {
        PreviousBirthsSectionAllOfChildren.Outcome.Livebirth.toTextModel().assertResourceModel(expected = R.string.child_outcome_livebirth)
        PreviousBirthsSectionAllOfChildren.Outcome.Stillbirth.toTextModel().assertResourceModel(expected = R.string.child_outcome_stillbirth)

        val nullable: PreviousBirthsSectionAllOfChildren.Outcome? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun previousBirthsSectionAllOfBirthsSubjectiveBirthExperienceToTextModel() {
        PreviousBirthsSectionAllOfBirths.SubjectiveBirthExperience.Bad.toTextModel().assertResourceModel(expected = R.string.subjective_experience_bad)
        PreviousBirthsSectionAllOfBirths.SubjectiveBirthExperience.Good.toTextModel().assertResourceModel(expected = R.string.subjective_experience_good)
        PreviousBirthsSectionAllOfBirths.SubjectiveBirthExperience.Normal.toTextModel().assertResourceModel(expected = R.string.subjective_experience_normal)

        val nullable: PreviousBirthsSectionAllOfBirths.SubjectiveBirthExperience? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun previousAbortionsSectionAllOfAbortionsTypeToTextModel() {
        PreviousAbortionsSectionAllOfAbortions.Type.Induced.toTextModel().assertResourceModel(expected = R.string.abortion_type_induced)
        PreviousAbortionsSectionAllOfAbortions.Type.Spontaneous.toTextModel().assertResourceModel(expected = R.string.abortion_type_spontaneous)

        val nullable: PreviousAbortionsSectionAllOfAbortions.Type? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun midwifeClinicSectionAllOfParentalCraftClassDesiredConsultationFormToTextModel() {
        MidwifeClinicSectionAllOfParentalCraftClassDesired.ConsultationForm.Group.toTextModel().assertResourceModel(expected = R.string.midwife_consultation_form_group)
        MidwifeClinicSectionAllOfParentalCraftClassDesired.ConsultationForm.Individual.toTextModel().assertResourceModel(expected = R.string.midwife_consultation_form_individual)

        val nullable: MidwifeClinicSectionAllOfParentalCraftClassDesired.ConsultationForm? = null
        nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
    }

    @Test
    fun socialHistorySectionMaritalOrPartnershipStatusToTextModel() {
        SocialHistorySection.MaritalOrPartnershipStatus.Single.toTextModel().assertResourceModel(expected = R.string.partnership_status_single)
        SocialHistorySection.MaritalOrPartnershipStatus.Married.toTextModel().assertResourceModel(expected = R.string.partnership_status_married)
        SocialHistorySection.MaritalOrPartnershipStatus.Separated.toTextModel().assertResourceModel(expected = R.string.partnership_status_separated)
        SocialHistorySection.MaritalOrPartnershipStatus.Divorced.toTextModel().assertResourceModel(R.string.partnership_status_divorced)
        SocialHistorySection.MaritalOrPartnershipStatus.Widowed.toTextModel().assertResourceModel(R.string.partnership_status_widowed)

        val nullable: SocialHistorySection.MaritalOrPartnershipStatus? = null
        nullable.toTextModel().assertResourceModel(R.string.common_missing_data)
    }

    @Test
    fun socialHistorySectionLivingConditionsToTextModel() {
        SocialHistorySection.LivingConditions.Partnership.toTextModel().assertResourceModel(R.string.living_conditions_partnership)
        SocialHistorySection.LivingConditions.Single.toTextModel().assertResourceModel(R.string.living_conditions_single)

        val nullable: SocialHistorySection.LivingConditions? = null
        nullable.toTextModel().assertResourceModel(R.string.common_missing_data)
    }

    @Test
    fun overallAssessmentSectionAllOfPrenatalCareAssessmentCareLevelToTextModel() {
        OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevel.Basic.toTextModel().assertResourceModel(R.string.prenatal_carelevel_basic)
        OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevel.Extended.toTextModel().assertResourceModel(R.string.prenatal_carelevel_extended)
        OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevel.HighRiskPregnancyCare.toTextModel().assertResourceModel(R.string.prenatal_carelevel_high_rish_pregnancy_care)
        OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevel.SpecialistMultidisciplinaryTeam.toTextModel().assertResourceModel(R.string.prenatal_carelevel_specialist_multidisciplinary_team)

        val nullable: OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevel? = null
        nullable.toTextModel().assertResourceModel(R.string.common_missing_data)
    }

    @Test
    fun riskAssessment3DownsScreeningProbabilityAssessmentToTextModel() {
        RiskAssessment3.DownsScreeningProbabilityAssessment.High.toTextModel().assertResourceModel(R.string.probability_assessment_high)
        RiskAssessment3.DownsScreeningProbabilityAssessment.Low.toTextModel().assertResourceModel(R.string.probability_assessment_low)

        val nullable: RiskAssessment3.DownsScreeningProbabilityAssessment? = null
        nullable.toTextModel().assertResourceModel(R.string.common_missing_data)
    }

    @Test
    fun midwifeConsultationSectionAllOfMidwifeConsultationWishesWeekdayToTextModel() {
        MidwifeConsultationSectionAllOfMidwifeConsultationWishes.Weekday.Monday.toTextModel().assertResourceModel(R.string.midwife_weekday_wish_monday)
        MidwifeConsultationSectionAllOfMidwifeConsultationWishes.Weekday.Tuesday.toTextModel().assertResourceModel(R.string.midwife_weekday_wish_tuesday)
        MidwifeConsultationSectionAllOfMidwifeConsultationWishes.Weekday.Wednesday.toTextModel().assertResourceModel(R.string.midwife_weekday_wish_wednesday)
        MidwifeConsultationSectionAllOfMidwifeConsultationWishes.Weekday.Thursday.toTextModel().assertResourceModel(R.string.midwife_weekday_wish_thursday)
        MidwifeConsultationSectionAllOfMidwifeConsultationWishes.Weekday.Friday.toTextModel().assertResourceModel(R.string.midwife_weekday_wish_friday)
        MidwifeConsultationSectionAllOfMidwifeConsultationWishes.Weekday.Saturday.toTextModel().assertResourceModel(R.string.midwife_weekday_wish_saturday)
        MidwifeConsultationSectionAllOfMidwifeConsultationWishes.Weekday.Sunday.toTextModel().assertResourceModel(R.string.midwife_weekday_wish_sunday)

        val nullable: MidwifeConsultationSectionAllOfMidwifeConsultationWishes.Weekday? = null
        nullable.toTextModel().assertResourceModel(R.string.common_missing_data)
    }

    class ListWorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRiskTypesToCommaSeparatedTextModel {
        @Test
        fun multiple() {
            listOf(
                WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types.Biological,
                WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types.Ergonomic,
                WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types.Chemical,
            ).toCommaSeparatedTextModel().assertIsAndApply<TextCompositeModel> {
                seperator.assertStringModel(", ")
                bindings.size.assertEquals(3)

                bindings.getOrNull(0).assertResourceModel(R.string.work_environment_risk_type_biological)
                bindings.getOrNull(1).assertResourceModel(R.string.work_environment_risk_type_ergonomic)
                bindings.getOrNull(2).assertResourceModel(R.string.work_environment_risk_type_chemical)
            }
        }

        @Test
        fun single() {
            listOf(
                WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types.Other,
            ).toCommaSeparatedTextModel().assertIsAndApply<TextCompositeModel> {
                seperator.assertStringModel(", ")
                bindings.size.assertEquals(1)

                bindings.getOrNull(0).assertResourceModel(R.string.work_environment_risk_type_other)
            }
        }

        @Test
        fun empty() {
            emptyList<WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types>().toCommaSeparatedTextModel()
                .assertResourceModel(R.string.common_missing_data)
        }

        @Test
        fun nullList() {
            val nullable: List<WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types>? = null
            nullable.toCommaSeparatedTextModel()
                .assertResourceModel(R.string.common_missing_data)
        }
    }
}
