/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.journal.extensions

import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.testextensions.assertResourceModel
import dk.nsp.models.DoubleValueWithUnitEnum
import dk.nsp.models.IntegerValueWithUnitEnum
import dk.nsp.models.NumericValueWithUnit
import dk.nsp.models.RangeValueWithUnit
import dk.nsp.models.UnitEnum
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIsAndApply
import dk.nsp.util.textmodel.TextCompositeModel
import dk.nsp.util.textmodel.TextStringModel
import org.junit.jupiter.api.Test
import java.math.BigDecimal

class ValueUnitExtensionsTest {

    class IntegerValueWithUnitEnumToTextModel {
        @Test
        fun full() {
            IntegerValueWithUnitEnum(
                10,
                UnitEnum.Kg,
            ).toTextModel().assertIsAndApply<TextCompositeModel> {
                seperator.assertIsAndApply<TextStringModel> {
                    text.assertEquals(" ")
                }

                bindings.size.assertEquals(2)
                bindings.getOrNull(0).assertIsAndApply<TextStringModel> {
                    text.assertEquals("10")
                }
                bindings.getOrNull(1).assertIsAndApply<TextStringModel> {
                    text.assertEquals("kg")
                }
            }
        }

        @Test
        fun onlyValue() {
            IntegerValueWithUnitEnum(
                15,
                null,
            ).toTextModel().assertIsAndApply<TextStringModel> {
                text.assertEquals("15")
            }
        }

        @Test
        fun onlyUnit() {
            IntegerValueWithUnitEnum(
                null,
                UnitEnum.Cm,
            ).toTextModel().assertResourceModel(expected = R.string.common_missing_data)
        }

        @Test
        fun nulls() {
            IntegerValueWithUnitEnum(
                null,
                null,
            ).toTextModel().assertResourceModel(expected = R.string.common_missing_data)

            val nullable: IntegerValueWithUnitEnum? = null
            nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
        }
    }

    class DoubleValueWithUnitEnumToTextModel {
        @Test
        fun full() {
            DoubleValueWithUnitEnum(
                8.48,
                UnitEnum.D,
            ).toTextModel().assertIsAndApply<TextCompositeModel> {
                seperator.assertIsAndApply<TextStringModel> {
                    text.assertEquals(" ")
                }

                bindings.size.assertEquals(2)
                bindings.getOrNull(0).assertIsAndApply<TextStringModel> {
                    text.assertEquals("8.48")
                }
                bindings.getOrNull(1).assertIsAndApply<TextStringModel> {
                    text.assertEquals("d")
                }
            }
        }

        @Test
        fun onlyValue() {
            DoubleValueWithUnitEnum(
                10.3,
                null,
            ).toTextModel().assertIsAndApply<TextStringModel> {
                text.assertEquals("10.3")
            }
        }

        @Test
        fun onlyUnit() {
            DoubleValueWithUnitEnum(
                null,
                UnitEnum.W,
            ).toTextModel().assertResourceModel(expected = R.string.common_missing_data)
        }

        @Test
        fun nulls() {
            DoubleValueWithUnitEnum(
                null,
                null,
            ).toTextModel().assertResourceModel(expected = R.string.common_missing_data)

            val nullable: DoubleValueWithUnitEnum? = null
            nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
        }
    }

    class DoubleValueWithUnitEnumToTextModelWithoutUnit {
        @Test
        fun withValue() {
            DoubleValueWithUnitEnum(
                10.34,
                null,
            ).toTextModelWithoutUnit().assertIsAndApply<TextStringModel> {
                text.assertEquals("10.34")
            }
        }

        @Test
        fun nulls() {
            DoubleValueWithUnitEnum(
                null,
                null,
            ).toTextModelWithoutUnit().assertResourceModel(expected = R.string.common_missing_data)

            val nullable: DoubleValueWithUnitEnum? = null
            nullable.toTextModelWithoutUnit().assertResourceModel(expected = R.string.common_missing_data)
        }
    }

    class NumericValueWithUnitToTextModel {
        @Test
        fun full() {
            NumericValueWithUnit(
                BigDecimal.valueOf(10.8),
                "units",
            ).toTextModel().assertIsAndApply<TextCompositeModel> {
                seperator.assertIsAndApply<TextStringModel> {
                    text.assertEquals(" ")
                }

                bindings.size.assertEquals(2)
                bindings.getOrNull(0).assertIsAndApply<TextStringModel> {
                    text.assertEquals("10.8")
                }
                bindings.getOrNull(1).assertIsAndApply<TextStringModel> {
                    text.assertEquals("units")
                }
            }
        }

        @Test
        fun onlyValue() {
            NumericValueWithUnit(
                BigDecimal.valueOf(700),
                null,
            ).toTextModel().assertIsAndApply<TextStringModel> {
                text.assertEquals("700")
            }
        }

        @Test
        fun onlyUnit() {
            NumericValueWithUnit(
                null,
                "rounds",
            ).toTextModel().assertResourceModel(expected = R.string.common_missing_data)
        }

        @Test
        fun nulls() {
            NumericValueWithUnit(
                null,
                null,
            ).toTextModel().assertResourceModel(expected = R.string.common_missing_data)

            val nullable: NumericValueWithUnit? = null
            nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
        }
    }

    class RangeValueWithUnitToTextModel {
        @Test
        fun full() {
            RangeValueWithUnit(
                "10-100",
                "Okser",
            ).toTextModel().assertIsAndApply<TextCompositeModel> {
                seperator.assertIsAndApply<TextStringModel> {
                    text.assertEquals(" ")
                }

                bindings.size.assertEquals(2)
                bindings.getOrNull(0).assertIsAndApply<TextStringModel> {
                    text.assertEquals("10-100")
                }
                bindings.getOrNull(1).assertIsAndApply<TextStringModel> {
                    text.assertEquals("Okser")
                }
            }
        }

        @Test
        fun onlyValue() {
            RangeValueWithUnit(
                "15-15",
                null,
            ).toTextModel().assertIsAndApply<TextStringModel> {
                text.assertEquals("15-15")
            }
        }

        @Test
        fun onlyUnit() {
            RangeValueWithUnit(
                null,
                "minutter",
            ).toTextModel().assertResourceModel(expected = R.string.common_missing_data)
        }

        @Test
        fun nulls() {
            RangeValueWithUnit(
                null,
                null,
            ).toTextModel().assertResourceModel(expected = R.string.common_missing_data)

            val nullable: RangeValueWithUnit? = null
            nullable.toTextModel().assertResourceModel(expected = R.string.common_missing_data)
        }
    }
}
