/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.journal.sections.sectiondetails.personaldata

import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.testData.PersonalDataSectionTemporaryAddressEmpty
import dk.nsp.mingraviditet.testextensions.assertResourceModel
import dk.nsp.mingraviditet.testextensions.assertStringModel
import dk.nsp.models.PersonalDataSectionTemporaryAddress
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIsAndApply
import dk.nsp.testassertions.assertNotNullAndEquals
import dk.nsp.testassertions.assertNull
import dk.nsp.util.textmodel.TextCompositeModel
import org.junit.jupiter.api.Test
import java.time.LocalDateTime

class PersonalDataBllTest {

    class PersonalDataSectionTemporaryAddressZipAndCity {
        @Test
        fun withZipAndCity() {
            PersonalDataSectionTemporaryAddressEmpty.copy(
                zip = "8000",
                city = "Aarhus",
            ).zipAndCity().assertNotNullAndEquals("8000 Aarhus")
        }

        @Test
        fun withOnlyZip() {
            PersonalDataSectionTemporaryAddressEmpty.copy(
                zip = "8000",
                city = null,
            ).zipAndCity().assertNotNullAndEquals("8000")
        }

        @Test
        fun withOnlyCity() {
            PersonalDataSectionTemporaryAddressEmpty.copy(
                zip = null,
                city = "Aarhus",
            ).zipAndCity().assertNotNullAndEquals("Aarhus")
        }

        @Test
        fun onlyNulls() {
            PersonalDataSectionTemporaryAddressEmpty.copy(
                zip = null,
                city = null,
            ).zipAndCity().assertNull()
        }
    }

    class PersonalDataSectionTemporaryAddressGetFullAddressAsSingleLineTextModel {
        @Test
        fun withStreetZipAndCity() {
            PersonalDataSectionTemporaryAddressEmpty.copy(
                streetAndNumber = "Testgade 2",
                zip = "8000",
                city = "Aarhus",
            ).getFullAddressAsSingleLineTextModel()
                .assertStringModel("Testgade 2, 8000 Aarhus")
        }

        @Test
        fun withOnlyStreet() {
            PersonalDataSectionTemporaryAddressEmpty.copy(
                streetAndNumber = "Testgade 2",
                zip = null,
                city = null,
            ).getFullAddressAsSingleLineTextModel()
                .assertStringModel("Testgade 2")
        }

        @Test
        fun withOnlyZipAndCity() {
            PersonalDataSectionTemporaryAddressEmpty.copy(
                streetAndNumber = null,
                zip = "8000",
                city = "Aarhus",
            ).getFullAddressAsSingleLineTextModel()
                .assertStringModel("8000 Aarhus")
        }

        @Test
        fun onlyNulls() {
            PersonalDataSectionTemporaryAddressEmpty.copy(
                streetAndNumber = null,
                zip = null,
                city = null,
            ).getFullAddressAsSingleLineTextModel()
                .assertResourceModel(R.string.common_missing_data)
        }
    }

    class PersonalDataSectionTemporaryAddressGetDurationTextModel {
        @Test
        fun withValidToAndFrom() {
            PersonalDataSectionTemporaryAddress(
                validFrom = LocalDateTime.of(2020, 1, 10, 8, 4),
                validTo = LocalDateTime.of(2021, 2, 20, 8, 4),
            ).getDurationTextModel().assertIsAndApply<TextCompositeModel> {
                seperator.assertStringModel(" - ")

                bindings.size.assertEquals(2)
                bindings.getOrNull(0).assertStringModel("10.01.2020")
                bindings.getOrNull(1).assertStringModel("20.02.2021")
            }
        }

        @Test
        fun withValidTo() {
            PersonalDataSectionTemporaryAddress(
                validFrom = null,
                validTo = LocalDateTime.of(2021, 2, 20, 8, 4),
            ).getDurationTextModel().assertIsAndApply<TextCompositeModel> {
                seperator.assertStringModel(" - ")

                bindings.size.assertEquals(1)
                bindings.getOrNull(0).assertStringModel("20.02.2021")
            }
        }

        @Test
        fun withValidFrom() {
            PersonalDataSectionTemporaryAddress(
                validFrom = LocalDateTime.of(2020, 1, 10, 8, 4),
                validTo = null,
            ).getDurationTextModel().assertIsAndApply<TextCompositeModel> {
                seperator.assertStringModel(" - ")

                bindings.size.assertEquals(1)
                bindings.getOrNull(0).assertStringModel("10.01.2020")
            }
        }

        @Test
        fun withOnlyNulls() {
            PersonalDataSectionTemporaryAddress(
                validFrom = null,
                validTo = null,
            ).getDurationTextModel().assertResourceModel(R.string.common_missing_data)
        }
    }
}
