/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.widget

import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.testextensions.assertResourceModel
import dk.nsp.mingraviditet.testextensions.assertStringModel
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIsAndApply
import dk.nsp.testassertions.assertNotNullAndApply
import dk.nsp.testassertions.assertNull
import org.junit.jupiter.api.Test
import java.time.LocalDateTime

class OverviewPregnancyStateTest {

    class WidgetControls {

        @Test
        fun StateNone() {
            val controls = OverviewPregnancyState.None.widgetControls

            controls.size.assertEquals(3)

            controls.getOrNull(0).assertNotNullAndApply {
                title.assertResourceModel(R.string.overview_week_title)
                type.assertIsAndApply<OverviewWidgetControlType.Value> {
                    text.assertResourceModel(R.string.common_missing_data)
                }
            }
            controls.getOrNull(1).assertNotNullAndApply {
                title.assertResourceModel(R.string.overview_month_title)
                type.assertIsAndApply<OverviewWidgetControlType.Value> {
                    text.assertResourceModel(R.string.common_missing_data)
                }
            }
            controls.getOrNull(2).assertNotNullAndApply {
                title.assertResourceModel(R.string.overview_trimester_title)
                type.assertIsAndApply<OverviewWidgetControlType.Value> {
                    text.assertResourceModel(R.string.common_missing_data)
                }
            }
        }

        class StateActiveAndWaitingForBirth {
            @Test
            fun withRelevantValues() {
                val action = {}

                val controls = OverviewPregnancyState.ActiveAndWaitingForBirth(
                    monthsIntoPregnancy = 6.6,
                    dueDate = null,
                    week = 36,
                    gestationAgeString = "gestation",
                    trimester = 2,
                    numberOfFetuses = null,
                    onDidGiveBirthPressed = action,
                ).widgetControls

                controls.size.assertEquals(4)

                controls.getOrNull(0).assertNotNullAndApply {
                    title.assertResourceModel(R.string.overview_week_title)
                    type.assertIsAndApply<OverviewWidgetControlType.DualValue> {
                        largeText.assertStringModel("36.")
                        smallText.assertStringModel("gestation")
                    }
                }
                controls.getOrNull(1).assertNotNullAndApply {
                    title.assertResourceModel(R.string.overview_month_title)
                    type.assertIsAndApply<OverviewWidgetControlType.Value> {
                        text.assertStringModel("7.")
                    }
                }
                controls.getOrNull(2).assertNotNullAndApply {
                    title.assertResourceModel(R.string.overview_trimester_title)
                    type.assertIsAndApply<OverviewWidgetControlType.Value> {
                        text.assertStringModel("2.")
                    }
                }
                controls.getOrNull(3).assertNotNullAndApply {
                    title.assertNull()
                    type.assertIsAndApply<OverviewWidgetControlType.TextAction> {
                        text.assertResourceModel(R.string.overview_did_give_birth_button_title)
                        callback.assertEquals(action)
                    }
                }
            }

            @Test
            fun withAllNulls() {
                val controls = OverviewPregnancyState.ActiveAndWaitingForBirth(
                    null,
                    null,
                    null,
                    null,
                    null,
                    null,
                    {},
                ).widgetControls

                controls.size.assertEquals(3)

                controls.getOrNull(0).assertNotNullAndApply {
                    title.assertResourceModel(R.string.overview_week_title)
                    type.assertIsAndApply<OverviewWidgetControlType.DualValue> {
                        largeText.assertResourceModel(R.string.common_missing_data)
                        smallText.assertResourceModel(R.string.common_missing_data)
                    }
                }
                controls.getOrNull(1).assertNotNullAndApply {
                    title.assertResourceModel(R.string.overview_month_title)
                    type.assertIsAndApply<OverviewWidgetControlType.Value> {
                        text.assertResourceModel(R.string.common_missing_data)
                    }
                }
                controls.getOrNull(2).assertNotNullAndApply {
                    title.assertResourceModel(R.string.overview_trimester_title)
                    type.assertIsAndApply<OverviewWidgetControlType.Value> {
                        text.assertResourceModel(R.string.common_missing_data)
                    }
                }
            }
        }

        @Test
        fun StateActiveAndGaveBirth() {
            val dateOfBirth = LocalDateTime.of(2021, 2, 3, 4, 5)
            val callback = { _: LocalDateTime? -> }

            val controls = OverviewPregnancyState.ActiveAndGaveBirth(
                dateOfBirth = dateOfBirth,
                null,
                presentDateOfBirthModal = callback,
            ).widgetControls

            controls.size.assertEquals(1)

            controls.getOrNull(0).assertNotNullAndApply {
                title.assertResourceModel(R.string.overview_date_of_birth)
                type.assertIsAndApply<OverviewWidgetControlType.DateValue> {
                    dateOfBirthCallback.assertEquals(callback)
                    date.assertEquals(dateOfBirth)
                }
            }
        }

        @Test
        fun StatePregnancyEnded() {
            OverviewPregnancyState.PregnancyEnded(null).widgetControls.size.assertEquals(0)
        }
    }
}
