/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import com.android.build.gradle.BaseExtension
import groovy.xml.XmlSlurper
import groovy.xml.slurpersupport.NodeChild
import org.gradle.api.GradleException
import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.tasks.testing.logging.TestExceptionFormat
import org.gradle.api.tasks.testing.logging.TestLogEvent
import org.gradle.kotlin.dsl.dependencies
import org.gradle.kotlin.dsl.extra
import org.gradle.testing.jacoco.plugins.JacocoPluginExtension
import org.gradle.testing.jacoco.tasks.JacocoReport
import java.io.File
import java.util.Locale
import kotlin.math.roundToInt

class JacocoReportsPlugin : Plugin<Project> {

    private val Project.android: BaseExtension
        get() = extensions.findByName("android") as? BaseExtension
            ?: error("Not an Android module: $name")

    private val Project.jacoco: JacocoPluginExtension
        get() = extensions.findByName("jacoco") as? JacocoPluginExtension
            ?: error("Not a Jacoco module: $name")

    private val excludedFiles = mutableSetOf(
        // App
        "**/dk/nsp/mingraviditet/databinding/*", // The autogenerated android XML binding implementation

        "**/dk/nsp/mingraviditet/databinding/*", // The autogenerated android XML binding implementation
        "**/dk/nsp/mingraviditet/di/", // Exclude Dependency Injection modules from test coverage
        // Excluding custom application, since it"s currently only debug and Firebase setup
        "**/dk/nsp//mingraviditet/App*",

        // Currently no UI testing, so exclude fragments and viewmodels, since they are mainly presentational components
        "**/**ViewModel*",
        "**/**Fragment*",

        // Android
        "**/androidx/databinding/*",
        "**/DataBinderMapperImpl*.*",
        "**/DataBindingTriggerClass*.*",
        "**/BR.*",
        "**/BuildConfig.*",
        // For some reason one of the autogenerated Android Navigation directions was found for coverage
        "**/dk/nsp/mingraviditet/JournalDirections*.*",

        // Dagger/Hilt (Mainly autogenerated classes for DI)
        "**/dagger/hilt/internal/*",
        "**/DaggerApp_*.*",
        "**/App_ComponentTreeDeps*",

        "**/hilt_aggregated_deps/*",
        "**/App_HiltComponents*.*",
        "**/Hilt_*",
        "**/*HiltModule*",

        // Shared
        "**/dk/nsp/models/*", // Exclude all the models generated by the OpenAPI script

        // Api
        "**/dk/nsp/api/**/**ControllerMock*",
        // These are wrappers around controller implementations
        "**/dk/nsp/api/BackendApiLayerMock*",
        "**/dk/nsp/api/BackendApiLayerImpl*",

        // Exclude models (No need for testing constructors and get/set for data classes)
        "**/dk/nsp/api/cms/models/*",
        "**/dk/nsp/api/auth/OpenIdCredentials.class",
        "**/dk/nsp/api/auth/AuthException*",

        // Repo
        // Exclude models (No need for testing constructors and get/set for data classes)
        "**/dk/nsp/domain/careplan/CareplanActivities.class",
        "**/dk/nsp/domain/SettingSubSectionUrl.class",
        "**/dk/nsp/models/ArticleSearchResult.class",
        "**/dk/nsp/models/DataCardWarningState.class",
        "**/dk/nsp/domain/models/SettingSubSectionUrl.class",
        "**/dk/nsp/domain/journal/observations/JournalSectionRow.class",
    )

    override fun apply(project: Project) {
        with(project) {
            if (project.name != "TestAssertion") {
                plugins.run { apply("jacoco") }

                jacocoAfterEvaluate()

                dependencies {
                    add("implementation", "org.jacoco:org.jacoco.core:0.8.10")
                    add("testImplementation", project(":TestAssertions"))
                }
            }
        }
    }

    private fun Project.jacocoAfterEvaluate() = afterEvaluate {
        val flavorName = "ProjectEnv"
        val buildTypeName = "debug"
        val sourceName = "${flavorName}${buildTypeName.capitalize(Locale.ENGLISH)}"
        val sourcePath = "$flavorName/$buildTypeName"

        val testTaskName = "test${sourceName.capitalize(Locale.ENGLISH)}UnitTest"

        registerCodeCoverageTask(
            testTaskName = testTaskName,
            sourceName = sourceName,
            sourcePath = sourcePath,
            flavorName = flavorName,
            buildTypeName = buildTypeName,
        )
    }

    private fun Project.registerCodeCoverageTask(
        testTaskName: String,
        sourceName: String,
        sourcePath: String,
        flavorName: String,
        buildTypeName: String,
    ) {
//        println("Report name = ${testTaskName}Coverage")
        tasks.register("customJacocoTestReport", JacocoReport::class.java) {
//        tasks.register("${testTaskName}Coverage", JacocoReport::class.java) {
            dependsOn(testTaskName)

            group = "Reporting"
            description = "Generate Jacoco coverage reports on the ${sourceName.capitalize(Locale.ENGLISH)} build."

            val javaDirectories = fileTree(
                "${project.buildDir}/intermediates/classes/$sourcePath",
            ) { exclude(excludedFiles) }

            val kotlinDirectories = fileTree(
                "${project.buildDir}/tmp/kotlin-classes/$sourcePath",
            ) { exclude(excludedFiles) }

            val coverageSrcDirectories = listOf(
                "src/main/java",
                "src/$flavorName/java",
                "src/$buildTypeName/java",
            )

            classDirectories.setFrom(files(javaDirectories, kotlinDirectories))
            additionalClassDirs.setFrom(files(coverageSrcDirectories))
            sourceDirectories.setFrom(files(coverageSrcDirectories))
            executionData.setFrom(
                files("${project.buildDir}/outputs/unit_test_coverage/${sourceName}UnitTest/${testTaskName}UnitTest.exec"),
//                files("${project.buildDir}/jacoco/${testTaskName}.exec")
            )

            reports {
                xml.required.set(false)
                csv.required.set(false)

                html.required.set(true)
                html.outputLocation.set(file("$buildDir/../../qa-reports/coverage-reports-versionName-versionCode/${project.name}"))

                println("Create dir ${"$buildDir/../../qa-reports/coverage-reports-versionName-versionCode/${project.name}"}")
//                    outputLocation.set(file("$buildDir/../../qa-reports/coverage-reports-versionName-versionCode/${project.name}"))
//                    outputLocation.set(file("$buildDir/../../jacocoThingy/${project.name}"))

//                jacoco.reportsDirectory.dir("$buildDir/../../jacocoThingy/${project.name}")
//                println("Output location: ${html.outputLocation.get()}")
            }

            project.android.testOptions {
                unitTests.all {
                    it.testLogging {
                        events(TestLogEvent.PASSED, TestLogEvent.SKIPPED, TestLogEvent.FAILED)
                        exceptionFormat = TestExceptionFormat.FULL
                    }
//                    reports.html.outputLocation.set(File("$buildDir/../../qa-reports/unit-tests-1.0.0-1337/${project.name}"))
                }
            }

            doLast {
//                jacocoTestReport("${testTaskName}Coverage")
                println("Wrote '${project.name}' coverage report to ${reports.html.outputLocation}/index.html")
            }
        }
    }

    @Suppress("UNCHECKED_CAST")
    private fun Project.jacocoTestReport(testTaskName: String) {
        val reportsDirectory = jacoco.reportsDirectory.asFile.get()
        println("Jacoco test report for $testTaskName - $reportsDirectory")

        val report = file("$buildDir/../../jacocoThingy/${project.name}")

        logger.lifecycle("Checking coverage results: $report")

        val metrics = report.extractTestsCoveredByType()
        val limits = project.extra["limits"] as Map<String, Double>

        val failures = metrics.filter { entry ->
            entry.value < limits[entry.key]!!
        }.map { entry ->
            "- ${entry.key} coverage rate is: ${entry.value}%, minimum is ${limits[entry.key]}%"
        }

        if (failures.isNotEmpty()) {
            logger.quiet("------------------ Code Coverage Failed -----------------------")
            failures.forEach { logger.quiet(it) }
            logger.quiet("---------------------------------------------------------------")
            throw GradleException("Code coverage failed")
        }

        logger.quiet("------------------ Code Coverage Success -----------------------")
        metrics.forEach { entry ->
            logger.quiet("- ${entry.key} coverage rate is: ${entry.value}%")
        }
        logger.quiet("---------------------------------------------------------------")
    }

    @Suppress("UNCHECKED_CAST")
    private fun File.extractTestsCoveredByType(): Map<String, Double> {
        val xmlReader = XmlSlurper().apply {
            setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false)
            setFeature("http://apache.org/xml/features/disallow-doctype-decl", false)
        }

        val counterNodes: List<NodeChild> = xmlReader
            .parse(this).parent()
            .children()
            .filter {
                (it as NodeChild).name() == "counter"
            } as List<NodeChild>

        return counterNodes.associate { nodeChild ->
            val type = nodeChild.attributes()["type"].toString().toLowerCase(Locale.ENGLISH)

            val covered = nodeChild.attributes()["covered"].toString().toDouble()
            val missed = nodeChild.attributes()["missed"].toString().toDouble()
            val percentage = ((covered / (covered + missed)) * 10000.0).roundToInt() / 100.0

            Pair(type, percentage)
        }
    }
}
