/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.knowledge

import android.os.Parcelable
import dk.nsp.api.cms.models.ChecklistDTO
import dk.nsp.api.cms.models.ChecklistItemDTO
import kotlinx.parcelize.Parcelize
import kotlin.math.round

@Parcelize
data class Checklist(
    val id: Int,
    val title: String,
    val description: String?,
    val items: List<ChecklistItem>,
) : Parcelable {
    fun checkedItems() = items.filter { it.value }

    fun completeness(): Float {
        return if (items.isNotEmpty()) {
            round((checkedItems().size.toFloat() / items.size) * 100)
        } else {
            0F
        }
    }

    fun completenessAsDivision(): String {
        return "${items.filter { it.value }.size}/${items.size}"
    }

    fun getPercentageAsString(): String {
        return "${completeness().toInt()}%"
    }
}

@Parcelize
data class ChecklistItem(
    val id: String,
    val title: String,
    val description: String?,
    val value: Boolean,
) : Parcelable

fun List<ChecklistDTO>.toDomainModel(listOfCheckedIds: List<String>?): List<Checklist> {
    return this.map {
        Checklist(
            it.id,
            it.title,
            it.text,
            it.items.map { item ->
                val isChecked = listOfCheckedIds?.contains(item.id) == true
                item.toDomainModel(isChecked)
            },
        )
    }
}

fun Checklist.toServerModel(): ChecklistDTO {
    return ChecklistDTO(id, title, description, items.map { it.toServerModel() })
}

fun ChecklistItemDTO.toDomainModel(checkedValue: Boolean): ChecklistItem {
    return ChecklistItem(id, title, text, checkedValue)
}

fun ChecklistItem.toServerModel(): ChecklistItemDTO {
    return ChecklistItemDTO(id, title, description)
}
