/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.repository

import android.content.Context
import dk.nsp.api.cms.models.RecommendedArticlesDTO
import dk.nsp.api.getBackendApiLayer
import dk.nsp.domain.articles.ArticleCategory
import dk.nsp.domain.articles.RecommendedArticleCategory
import dk.nsp.domain.articles.toDomainModel
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.mapSuccess
import dk.nsp.shared.mapSuccessWithTryOrFailed
import dk.nsp.shared.util.asyncDefault
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Deferred

class ArticlesRepository(context: Context) {

    private val cmsController = context.getBackendApiLayer().cmsController

    fun getArticleSections(scope: CoroutineScope): Deferred<ResultWrapper<List<ArticleCategory>>> =
        scope.asyncDefault {
            val categoriesRes = cmsController.getArticleSections(this).await()

            categoriesRes.mapSuccess { it ->
                it.map {
                    it.toDomainModel()
                }
            }
        }

    fun CoroutineScope.getAfterBirthArcticleCategories(
        week: Int,
    ): Deferred<ResultWrapper<RecommendedArticleCategory>> = asyncDefault {
        val articlesRes = cmsController
            .getAfterBirthArticleCategories(this, week).await()
        mapArticleCategoryRes(articlesRes)
    }

    fun CoroutineScope.getArticleCategories(
        week: Int,
    ): Deferred<ResultWrapper<RecommendedArticleCategory>> = asyncDefault {
        val articlesRes = cmsController.getArticleCategories(scope = this, week = week).await()
        mapArticleCategoryRes(articlesRes)
    }

    private fun mapArticleCategoryRes(articlesRes: ResultWrapper<RecommendedArticlesDTO>): ResultWrapper<RecommendedArticleCategory> =
        articlesRes.mapSuccessWithTryOrFailed { categories ->
            categories.toDomainModel()
        }

    fun searchArticles(
        scope: CoroutineScope,
        query: String,
    ): Deferred<ResultWrapper<List<ArticleCategory>>> = scope.asyncDefault {
        val searchResponse = cmsController.searchArticles(this, query).await()

        searchResponse.mapSuccessWithTryOrFailed { categories ->
            categories.map { it.toDomainModel() }
        }
    }
}
