/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.repository

import android.content.Context
import dk.nsp.api.getBackendApiLayer
import dk.nsp.domain.dictionary.DictionaryEntry
import dk.nsp.domain.dictionary.toDomainModel
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.shared.util.asyncDefault
import dk.nsp.util.textmodel.TextModel
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.runBlocking

class DictionaryRepository(context: Context) {
    private val cmsController = context.getBackendApiLayer().cmsController

    private var _cachedDictionary = listOf<DictionaryEntry>()

    private var dictGotten = false

    private val resources = context.resources

    fun getDictionaryEntryFor(scope: CoroutineScope, key: TextModel): DictionaryEntry? {
        // if no dictionary we get it and call ourselves
        val defferedRes = scope.asyncDefault {
            if (_cachedDictionary.isEmpty() && !dictGotten) {
                getDictionary(scope)

                getDictionaryEntryFor(scope, key)
            } else if (dictGotten && _cachedDictionary.isNotEmpty()) {
                _cachedDictionary.firstOrNull { it.title == key.getString(resources) }
            } else {
                null
            }
        }

        return runBlocking {
            defferedRes.await()
        }
    }

    suspend fun getDictionary(scope: CoroutineScope): List<DictionaryEntry>? {
        val dict = when (val dictionaryRes = cmsController.getDictionary(scope).await()) {
            is Success -> {
                val domainModels = dictionaryRes.value.map { it.toDomainModel() }

                _cachedDictionary = domainModels
                dictGotten = true

                domainModels
            }
            is Failed -> {
                dictGotten = true
                null
            }
        }
        return dict
    }
}
