/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.util.textmodel

import android.content.res.Resources
import android.os.Parcelable
import androidx.annotation.PluralsRes
import androidx.annotation.StringRes

sealed class TextModel : Parcelable {
    companion object {
        @JvmStatic
        fun from(@StringRes text: Int): TextResourceModel {
            return TextResourceModel(text)
        }

        @JvmStatic
        fun from(@StringRes text: Int, vararg placeholders: TextModel): TextResourcePlaceholderModel {
            return TextResourcePlaceholderModel(text, placeholders)
        }

        @JvmStatic
        fun from(text: String): TextStringModel {
            return TextStringModel(text)
        }

        /**
         * Used when working with Quantity/Plural string resources
         * @param pluralStringRes Int - The resource ID for the plurals. The value 0 is an invalid identifier.
         * @param quantity Int - The number used to get the correct string for the current language's plural rules
         * @param placeholderValues Array<out String> Only use this parameter if the supplied [pluralStringRes] contains placeholders (see [Resources.getQuantityString] with 3 parameters for more info)
         * @return TextResourceQuantityBindingModel
         */
        fun from(
            @PluralsRes pluralStringRes: Int,
            quantity: Int,
            vararg placeholderValues: TextModel,
        ): TextResourceQuantityModel {
            return TextResourceQuantityModel(
                pluralStringRes,
                quantity,
                if (placeholderValues.isNotEmpty()) {
                    placeholderValues
                } else {
                    null
                },
            )
        }

        fun from(
            bindings: List<TextModel>,
            seperator: TextModel? = null,
        ): TextCompositeModel = TextCompositeModel(bindings, seperator)
    }

    abstract fun getString(resources: Resources): String?

    abstract override fun equals(other: Any?): Boolean
    override fun hashCode(): Int {
        return javaClass.hashCode()
    }
}
