/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.knowledge

import dk.nsp.api.cms.models.ChecklistDTO
import dk.nsp.api.cms.models.ChecklistItemDTO
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertFalse
import dk.nsp.testassertions.assertNotNull
import dk.nsp.testassertions.assertNotNullAndEquals
import dk.nsp.testassertions.assertTrue
import org.junit.jupiter.api.Nested
import org.junit.jupiter.api.Test

class ChecklistTest {

    private val nonCheckedChecklistItems = mutableListOf(
        ChecklistItem("0", "1", null, false),
        ChecklistItem("1", "2", null, false),
        ChecklistItem("2", "3", null, false),
        ChecklistItem("3", "4", null, false),
    )

    private val oneCheckedChecklistItems = mutableListOf(
        ChecklistItem("0", "1", null, true),
        ChecklistItem("1", "2", null, false),
        ChecklistItem("2", "3", null, false),
        ChecklistItem("3", "4", null, false),
    )

    private val sixChecklistItems = mutableListOf(
        ChecklistItem("0", "1", null, true),
        ChecklistItem("1", "2", null, false),
        ChecklistItem("2", "3", null, false),
        ChecklistItem("3", "4", null, false),
        ChecklistItem("4", "4", null, false),
        ChecklistItem("5", "4", null, false),
    )

    private val emptyCheckList =
        Checklist(0, "Empty items list", "Description", emptyList())
    private val noCheckItemsInChecklist =
        Checklist(0, "Test Checklist", "Test description", nonCheckedChecklistItems)
    private val oneCheckedItemInChecklist =
        Checklist(1, "Checklist with one item selected", null, oneCheckedChecklistItems)
    private val sixCheckedItemsInChecklist =
        Checklist(2, "Checklist with six items", null, sixChecklistItems)

    private val basicCheckListItemDto = ChecklistItemDTO(
        id = "ID",
        title = "Title",
        text = "Text",
    )

    private val basicCheckListDto = ChecklistDTO(
        id = 1,
        title = "Title",
        text = "Text",
        items = listOf(basicCheckListItemDto),
    )

    @Test
    fun checklistToServerModel() {
        val source = Checklist(
            id = 10,
            title = "Title",
            description = "Description",
            items = oneCheckedChecklistItems,
        )
        val result = source.toServerModel()

        result.id.assertEquals(source.id)
        result.title.assertEquals(source.title)
        result.text.assertEquals(source.description)

        result.items.let {
            it.size.assertEquals(4)

            it.getOrNull(0)?.id?.assertNotNullAndEquals("0")
            it.getOrNull(1)?.id?.assertNotNullAndEquals("1")
            it.getOrNull(2)?.id?.assertNotNullAndEquals("2")
            it.getOrNull(3)?.id?.assertNotNullAndEquals("3")
        }
    }

    @Test
    fun checklistItemDTOToDomainModelCheckedValue() {
        val source = ChecklistItemDTO(
            id = "ID",
            title = "Title",
            text = "Text",
        )
        val resultChecked = source.toDomainModel(true)
        val resultUnchecked = source.toDomainModel(false)

        resultChecked.value.assertTrue()
        resultUnchecked.value.assertFalse()
    }

    @Test
    fun checklistItemToServerModel() {
        val source = ChecklistItem(
            "id",
            "Title",
            "Description",
            false,
        )
        val result = source.toServerModel()

        result.id.assertEquals(source.id)
        result.title.assertEquals(source.title)
        result.text.assertEquals(source.description)
    }

    @Test
    fun completeness() {
        noCheckItemsInChecklist.completeness().assertEquals(0f)
        oneCheckedItemInChecklist.completeness().assertEquals(25F)
        sixCheckedItemsInChecklist.completeness().assertEquals(17F)

        emptyCheckList.completeness().assertEquals(0f)
    }

    @Test
    fun completenessAsDivision() {
        noCheckItemsInChecklist.completenessAsDivision().assertEquals("0/4")
        oneCheckedItemInChecklist.completenessAsDivision().assertEquals("1/4")
        sixCheckedItemsInChecklist.completenessAsDivision().assertEquals("1/6")
    }

    @Test
    fun getPercentageAsString() {
        noCheckItemsInChecklist.getPercentageAsString().assertEquals("0%")
        oneCheckedItemInChecklist.getPercentageAsString().assertEquals("25%")
        sixCheckedItemsInChecklist.getPercentageAsString().assertEquals("17%")
    }

    @Test
    fun checkedItems() {
        val checkedItems = Checklist(
            1,
            "Title",
            "Description",
            listOf(
                ChecklistItem("0", "1", null, true),
                ChecklistItem("1", "2", null, false),
                ChecklistItem("2", "3", null, true),
                ChecklistItem("3", "4", null, false),
            ),
        ).checkedItems()

        checkedItems.size.assertEquals(2)
        checkedItems.getOrNull(0)?.id?.assertNotNullAndEquals("0")
        checkedItems.getOrNull(1)?.id?.assertNotNullAndEquals("2")
    }

    @Nested
    inner class ListChecklistDTOToDomainModelListOfCheckedIds {

        val source = listOf(
            basicCheckListDto.copy(
                id = 10,
                items = listOf(
                    basicCheckListItemDto.copy(id = "1"),
                    basicCheckListItemDto.copy(id = "2"),
                ),
            ),
            basicCheckListDto.copy(
                id = 11,
                items = listOf(
                    basicCheckListItemDto.copy(id = "3"),
                    basicCheckListItemDto.copy(id = "4"),
                ),
            ),
            basicCheckListDto.copy(
                id = 12,
                items = listOf(
                    basicCheckListItemDto.copy(id = "5"),
                    basicCheckListItemDto.copy(id = "6"),
                    basicCheckListItemDto.copy(id = "7"),
                ),
            ),
        )

        @Test
        fun withCheckedList() {
            val checkedItems = listOf("2", "5", "7")
            val result = source.toDomainModel(checkedItems)

            result.size.assertEquals(3)

            result.getOrNull(0).let { checklist ->
                checklist.assertNotNull()
                checklist.id.assertEquals(10)
                checklist.items.let { items ->
                    items.size.assertEquals(2)

                    // Assert correct items are checked
                    items.getOrNull(0)?.value.assertNotNullAndEquals(false)
                    items.getOrNull(1)?.value.assertNotNullAndEquals(true)
                }
            }

            result.getOrNull(1).let { checklist ->
                checklist.assertNotNull()
                checklist.id.assertEquals(11)
                checklist.items.let { items ->
                    items.size.assertEquals(2)

                    // Assert correct items are checked
                    items.getOrNull(0)?.value.assertNotNullAndEquals(false)
                    items.getOrNull(1)?.value.assertNotNullAndEquals(false)
                }
            }

            result.getOrNull(2).let { checklist ->
                checklist.assertNotNull()
                checklist.id.assertEquals(12)
                checklist.items.let { items ->
                    items.size.assertEquals(3)

                    // Assert correct items are checked
                    items.getOrNull(0)?.value.assertNotNullAndEquals(true)
                    items.getOrNull(1)?.value.assertNotNullAndEquals(false)
                    items.getOrNull(2)?.value.assertNotNullAndEquals(true)
                }
            }
        }

        @Test
        fun withNullCheckedList() {
            val result = source.toDomainModel(null)

            result.forEach { checklist ->
                checklist.items.forEach { item ->
                    item.value.assertFalse()
                }
            }
        }
    }
}
