/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.repository

import android.content.Context
import dk.nsp.api.IBackendApiLayer
import dk.nsp.api.cms.ICMSController
import dk.nsp.api.cms.models.ArticleCategoryDTO
import dk.nsp.api.cms.models.ArticleContentSectionDTO
import dk.nsp.api.cms.models.ArticleDTO
import dk.nsp.api.cms.models.RecommendedArticlesDTO
import dk.nsp.domain.articles.ArticleCategory
import dk.nsp.domain.articles.RecommendedArticleCategory
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.shared.toSuccess
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIs
import dk.nsp.testassertions.assertIsAndApply
import dk.nsp.testassertions.assertNotNullAndApply
import io.mockk.coEvery
import io.mockk.every
import io.mockk.mockk
import io.mockk.mockkObject
import kotlinx.coroutines.runBlocking
import org.junit.jupiter.api.Test
import java.time.LocalDateTime

class ArticlesRepositoryTest {

    val defaultArticleDtosSuccess = listOf(
        ArticleCategoryDTO(
            1,
            "1",
            "",
            null,
            articles = listOf(
                ArticleDTO(
                    id = 11,
                    "",
                    null,
                    LocalDateTime.now(),
                    "",
                    content = "Content",
                    sections = listOf(
                        ArticleContentSectionDTO(
                            id = 111,
                            "111",
                            null,
                            "",
                            null,
                        ),
                    ),
                ),
            ),
            "",
        ),
        ArticleCategoryDTO(
            2,
            "2",
            "",
            null,
            articles = listOf(
                ArticleDTO(
                    id = 22,
                    "",
                    null,
                    LocalDateTime.now(),
                    "",
                    content = "Content",
                    sections = listOf(
                        ArticleContentSectionDTO(
                            id = 222,
                            "222",
                            null,
                            "",
                            null,
                        ),
                    ),
                ),
            ),
            "",
        ),
    ).toSuccess()

    val defaultRecommendedArticleDtoSuccess = RecommendedArticlesDTO(
        currentWeek = defaultArticleDtosSuccess.value,
        nextWeek = emptyList(),
    ).toSuccess()

    private val mockedController = mockk<ICMSController> {
        coEvery {
            getArticleSections(any()).await()
        } returns defaultArticleDtosSuccess

        coEvery {
            getAfterBirthArticleCategories(any(), any()).await()
        } returns defaultRecommendedArticleDtoSuccess

        coEvery {
            getArticleCategories(any(), any()).await()
        } returns defaultRecommendedArticleDtoSuccess

        coEvery {
            searchArticles(any(), any()).await()
        } returns defaultArticleDtosSuccess
    }

    private val failingController = mockk<ICMSController> {
        coEvery {
            getArticleSections(any()).await()
        } returns Failed(Exception("failed to get article sections"))

        coEvery {
            getAfterBirthArticleCategories(any(), any()).await()
        } returns Failed(Exception("failed to get after birth"))

        coEvery {
            getArticleCategories(any(), any()).await()
        } returns Failed(Exception("failed to article categories"))

        coEvery {
            searchArticles(any(), any()).await()
        } returns Failed(Exception("failed to search"))
    }

    private val mockBackend = mockk<IBackendApiLayer> {
        every {
            cmsController
        } returns mockedController
    }

    private val failingBackend = mockk<IBackendApiLayer> {
        every {
            cmsController
        } returns failingController
    }

    private fun getRepo(): ArticlesRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns mockBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return ArticlesRepository(mockedContext)
    }

    private fun getFailingRepo(): ArticlesRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns failingBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return ArticlesRepository(mockedContext)
    }

    @Test
    fun getArticleSections() = runBlocking {
        getRepo().getArticleSections(this)
            .await()
            .assertIsAndApply<Success<List<ArticleCategory>>> {
                value.size.assertEquals(2)
                value.getOrNull(0).assertNotNullAndApply {
                    name.assertEquals("1")
                    articles.getOrNull(0).assertNotNullAndApply {
                        id.assertEquals(11)
                        sections.getOrNull(0).assertNotNullAndApply {
                            id.assertEquals(111)
                        }
                    }
                }
                value.getOrNull(1).assertNotNullAndApply {
                    name.assertEquals("2")
                    articles.getOrNull(0).assertNotNullAndApply {
                        id.assertEquals(22)
                        sections.getOrNull(0).assertNotNullAndApply {
                            id.assertEquals(222)
                        }
                    }
                }
            }

        getFailingRepo().getArticleSections(this)
            .await()
            .assertIs<Failed>()
    }

    @Test
    fun getAfterBirthArcticleCategories() = runBlocking {
        with(getRepo()) { getAfterBirthArcticleCategories(0) }
            .await()
            .assertIsAndApply<Success<RecommendedArticleCategory>> {
                val value = this.value.currentWeek
                value.size.assertEquals(2)
                value.getOrNull(0).assertNotNullAndApply {
                    name.assertEquals("1")
                    articles.getOrNull(0).assertNotNullAndApply {
                        id.assertEquals(11)
                        sections.getOrNull(0).assertNotNullAndApply {
                            id.assertEquals(111)
                        }
                    }
                }
                value.getOrNull(1).assertNotNullAndApply {
                    name.assertEquals("2")
                    articles.getOrNull(0).assertNotNullAndApply {
                        id.assertEquals(22)
                        sections.getOrNull(0).assertNotNullAndApply {
                            id.assertEquals(222)
                        }
                    }
                }
            }

        with(getFailingRepo()) { getAfterBirthArcticleCategories(0) }
            .await()
            .assertIs<Failed>()
    }

    @Test
    fun getArticleCategories() = runBlocking {
        with(getRepo()) { getArticleCategories(0) }
            .await()
            .assertIsAndApply<Success<RecommendedArticleCategory>> {
                val value = this.value.currentWeek
                value.size.assertEquals(2)
                value.getOrNull(0).assertNotNullAndApply {
                    name.assertEquals("1")
                    articles.getOrNull(0).assertNotNullAndApply {
                        id.assertEquals(11)
                        sections.getOrNull(0).assertNotNullAndApply {
                            id.assertEquals(111)
                        }
                    }
                }
                value.getOrNull(1).assertNotNullAndApply {
                    name.assertEquals("2")
                    articles.getOrNull(0).assertNotNullAndApply {
                        id.assertEquals(22)
                        sections.getOrNull(0).assertNotNullAndApply {
                            id.assertEquals(222)
                        }
                    }
                }
            }

        with(getFailingRepo()) { getArticleCategories(0) }
            .await()
            .assertIs<Failed>()
    }

    @Test
    fun searchArticles() = runBlocking {
        getRepo().searchArticles(this, "")
            .await()
            .assertIsAndApply<Success<List<ArticleCategory>>> {
                value.size.assertEquals(2)
                value.getOrNull(0).assertNotNullAndApply {
                    name.assertEquals("1")
                    articles.getOrNull(0).assertNotNullAndApply {
                        id.assertEquals(11)
                        sections.getOrNull(0).assertNotNullAndApply {
                            id.assertEquals(111)
                        }
                    }
                }
                value.getOrNull(1).assertNotNullAndApply {
                    name.assertEquals("2")
                    articles.getOrNull(0).assertNotNullAndApply {
                        id.assertEquals(22)
                        sections.getOrNull(0).assertNotNullAndApply {
                            id.assertEquals(222)
                        }
                    }
                }
            }

        getFailingRepo().searchArticles(this, "")
            .await()
            .assertIs<Failed>()
    }
}
