/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.repository

import android.content.Context
import dk.nsp.api.IBackendApiLayer
import dk.nsp.api.careplan.ICareplanController
import dk.nsp.domain.testutils.InstantExecutorExtension
import dk.nsp.models.Activity
import dk.nsp.models.CarePlan
import dk.nsp.models.CarePlanResponsibleOrganization
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.shared.toSuccess
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIs
import dk.nsp.testassertions.assertIsAndApply
import dk.nsp.testassertions.assertNotNull
import dk.nsp.testassertions.assertNull
import io.mockk.coEvery
import io.mockk.every
import io.mockk.mockk
import io.mockk.mockkObject
import kotlinx.coroutines.runBlocking
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.extension.ExtendWith

@ExtendWith(InstantExecutorExtension::class)
class CareplanRepositoryTest {

    private val mockedCareplanController = mockk<ICareplanController> {
        coEvery {
            getCareplan(any(), any()).await()
        } returns CarePlan(
            "Patient1",
            "Title",
            CarePlanResponsibleOrganization(),
            activities = listOf(
                Activity(
                    Activity.Status.Finished,
                    "Title1",
                    id = "ID1",
                ),
                Activity(
                    Activity.Status.Finished,
                    "Title2",
                    id = "ID2",
                ),
            ),
        ).toSuccess()

        coEvery {
            getCareplan(any(), "emptyActivities").await()
        } returns CarePlan(
            "Patient1",
            "Title",
            CarePlanResponsibleOrganization(),
            activities = emptyList(),
        ).toSuccess()
    }

    private val failingCareplanController = mockk<ICareplanController> {
        coEvery {
            getCareplan(any(), any()).await()
        } returns Failed(Exception())
    }

    private val mockBackend = mockk<IBackendApiLayer> {
        every {
            careplanController
        } returns mockedCareplanController
    }

    private val failingBackend = mockk<IBackendApiLayer> {
        every {
            careplanController
        } returns failingCareplanController
    }

    private fun getRepo(): CareplanRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns mockBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return CareplanRepository(mockedContext)
    }

    private fun getFailingRepo(): CareplanRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns failingBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return CareplanRepository(mockedContext)
    }

    @Test
    fun getCareplans() = runBlocking {
        val repo = getRepo()
        val carePlan = repo
            .getCareplans(this, "")
            .await()
        carePlan.assertIsAndApply<Success<CarePlan>> {
            repo.selectedCareplan.value.assertEquals(this)
        }

        val failingRepo = getFailingRepo()

        val failedCareplan = failingRepo.getCareplans(this, "")
            .await()

        failedCareplan.assertIs<Failed>()

        failingRepo.selectedCareplan.value.assertEquals(failedCareplan)
    }

    @Test
    fun getCareplanActivity() = runBlocking {
        val repo = getRepo()

        repo.getCareplanActivity("ID1").assertNull()

        repo.selectedCareplan.value.assertNull()

        repo.getCareplans(this, "")
            .await()

        repo.getCareplanActivity("ID2")
            .assertNotNull()

        repo.getCareplanActivity("ID3")
            .assertNull()

        repo.getCareplans(this, "emptyActivities")
            .await()
        repo.getCareplanActivity("ID2").assertNull()
    }

    @Test
    fun getCarePlanActivityFailing() = runBlocking {
        val failingRepo = getFailingRepo()
        failingRepo.getCareplans(this, "").await()
        failingRepo.getCareplanActivity("").assertNull()
    }

    @Test
    fun clear() = runBlocking {
        val repo = getRepo()
        repo.selectedCareplan.value.assertNull()

        repo.getCareplans(this, "")
            .await()

        repo.selectedCareplan.value.assertNotNull()

        repo.clear()

        repo.selectedCareplan.value.assertNull()
    }
}
