/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.repository

import android.content.Context
import dk.nsp.api.IBackendApiLayer
import dk.nsp.api.cms.ICMSController
import dk.nsp.api.cms.models.ChecklistDTO
import dk.nsp.api.cms.models.ChecklistItemDTO
import dk.nsp.api.configuration.IConfigurationController
import dk.nsp.domain.knowledge.Checklist
import dk.nsp.domain.knowledge.ChecklistItem
import dk.nsp.domain.testutils.InstantExecutorExtension
import dk.nsp.models.CheckListItem
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.shared.asSuccessValueOrNull
import dk.nsp.shared.toSuccess
import dk.nsp.testassertions.*
import io.mockk.coEvery
import io.mockk.every
import io.mockk.mockk
import io.mockk.mockkObject
import kotlinx.coroutines.runBlocking
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.extension.ExtendWith

@ExtendWith(InstantExecutorExtension::class)
class ChecklistRepositoryTest {

    private val defaultChecklistData = listOf(
        ChecklistDTO(
            1,
            "First",
            null,
            items = listOf(
                ChecklistItemDTO(
                    "11",
                    "",
                    null,
                ),
                ChecklistItemDTO(
                    "12",
                    "",
                    null,
                ),
            ),
        ),
    )

    private val mockedCMSController = mockk<ICMSController> {
        coEvery {
            getChecklists(any()).await()
        } returns defaultChecklistData.toSuccess()
    }

    private val failingCMSController = mockk<ICMSController> {
        coEvery {
            getChecklists(any()).await()
        } returns Failed(Exception())
    }

    private val mockedConfigurationController = mockk<IConfigurationController> {
        coEvery {
            getCheckedChecklistItems(any(), any()).await()
        } returns listOf("12").toSuccess()

        coEvery {
            updateChecklistItem(any(), any(), any()).await()
        } returns CheckListItem("", true).toSuccess()
    }

    private val failingConfigurationController = mockk<IConfigurationController> {
        coEvery {
            getCheckedChecklistItems(any(), any()).await()
        } returns Failed(Exception())

        coEvery {
            updateChecklistItem(any(), any(), any()).await()
        } returns Failed(Exception())
    }

    private val mockBackend = mockk<IBackendApiLayer> {
        every {
            configurationController
        } returns mockedConfigurationController

        every {
            cmsController
        } returns mockedCMSController
    }

    private val failingBackend = mockk<IBackendApiLayer> {
        every {
            cmsController
        } returns failingCMSController

        every {
            configurationController
        } returns failingConfigurationController
    }

    private fun getRepo(): ChecklistRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns mockBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return ChecklistRepository(mockedContext)
    }

    private fun getFailingRepo(): ChecklistRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns failingBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return ChecklistRepository(mockedContext)
    }

    @Test
    fun getChecklists() = runBlocking {
        val repo = getRepo()
        val result = repo
            .getChecklists(this, "")
            .await()

        result.assertIsAndApply<Success<List<Checklist>>> {
            repo.checklists.value.assertEquals(value)

            value.size.assertEquals(1)
            value.getOrNull(0)
                .assertNotNullAndApply {
                    id.assertEquals(1)
                    items.size.assertEquals(2)

                    items.getOrNull(1).assertNotNullAndApply {
                        id.assertEquals("12")
                        value.assertTrue()
                    }
                }
        }

        getFailingRepo().getChecklists(this, "")
            .await()
            .assertIs<Failed>()
    }

    @Test
    fun clearChecklists() = runBlocking {
        val repo = getRepo()
        repo.getChecklists(this, "")
            .await()

        repo.checklists.value.assertNotNull()

        repo.clearChecklists(this)
            .await()

        repo.checklists.value.assertNull()
    }

    @Test
    fun updateChecklistItem() = runBlocking {
        val repo = getRepo()

        val checklist = repo.getChecklists(this, "").await().asSuccessValueOrNull()?.firstOrNull()
        checklist.assertNotNull()
        val firstItem = checklist.items.firstOrNull()
        firstItem.assertNotNull()

        val updateResult = repo.updateChecklistItem(
            this,
            "",
            checklist,
            firstItem,
        ).await()

        updateResult.assertIs<Success<Unit>>()

        val updatedChecklist = repo.checklists.value
        updatedChecklist.assertNotNull()
        updatedChecklist.firstOrNull().assertNotNullAndApply {
            items.firstOrNull().assertNotNullAndApply {
                value.assertTrue()
            }
        }

        getFailingRepo().updateChecklistItem(
            this,
            "",
            Checklist(1, "", null, emptyList()),
            ChecklistItem("", "", null, false),
        )
            .await()
            .assertIs<Failed>()
    }

    @Test
    fun clear() = runBlocking {
        val repo = getRepo()
        repo.getChecklists(this, "")
            .await()

        repo.checklists.value.assertNotNull()

        repo.clear()

        repo.checklists.value.assertNull()
    }
}
