/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.repository

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.Observer
import dk.nsp.api.IBackendApiLayer
import dk.nsp.api.configuration.IConfigurationController
import dk.nsp.api.datacard.IDataCardController
import dk.nsp.domain.testutils.InstantExecutorExtension
import dk.nsp.models.CprIdentifiedPerson
import dk.nsp.models.DataCard
import dk.nsp.models.DateOfBirth
import dk.nsp.models.PersonName
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.shared.toSuccess
import dk.nsp.testassertions.*
import io.mockk.coEvery
import io.mockk.every
import io.mockk.mockk
import io.mockk.mockkObject
import kotlinx.coroutines.runBlocking
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.extension.ExtendWith
import java.time.LocalDateTime

@ExtendWith(InstantExecutorExtension::class)
class DataCardRepositoryTest {

    private val mockedDataCardController = mockk<IDataCardController> {
        coEvery {
            getMaternityDataCard(any()).await()
        } returns listOf(
            DataCard(
                "EP1",
                CprIdentifiedPerson(PersonName(), "ID1"),
                documentStatus = DataCard.DocumentStatus.CLOSED,
            ),
            DataCard(
                "EP2",
                CprIdentifiedPerson(PersonName(), "ID2"),
                documentStatus = DataCard.DocumentStatus.ACTIVE,
            ),
        ).toSuccess()
    }

    private val mockedConfigurationController =
        mockk<IConfigurationController> {
            coEvery {
                getDateOfBirth(any(), "EP1").await()
            } returns DateOfBirth(
                LocalDateTime.of(
                    2020,
                    7,
                    6,
                    4,
                    1,
                ),
            ).toSuccess()

            coEvery {
                getDateOfBirth(any(), "EP2").await()
            } returns Success(
                DateOfBirth(
                    LocalDateTime.of(
                        2018,
                        7,
                        6,
                        4,
                        1,
                    ),
                ),
            )

            coEvery {
                updateDateOfBirth(
                    any(),
                    any(),
                    LocalDateTime.of(2021, 9, 2, 4, 3),
                ).await()
            } returns
                DateOfBirth(LocalDateTime.of(2021, 9, 2, 4, 3)).toSuccess()
        }

    private val failingDataCardController = mockk<IDataCardController> {
        coEvery {
            getMaternityDataCard(any()).await()
        } returns Failed(Exception())
    }

    private val failingConfigurationController =
        mockk<IConfigurationController> {
            coEvery {
                getDateOfBirth(any(), any()).await()
            } returns Failed(Exception())
        }

    private val mockBackend = mockk<IBackendApiLayer> {
        every {
            maternityDataCardController
        } returns mockedDataCardController

        every {
            configurationController
        } returns mockedConfigurationController
    }

    private val failingBackend = mockk<IBackendApiLayer> {
        every {
            maternityDataCardController
        } returns failingDataCardController

        every {
            configurationController
        } returns failingConfigurationController
    }

    private fun getRepo(): DataCardRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns mockBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return DataCardRepository(mockedContext)
    }

    private fun getFailingRepo(): DataCardRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns failingBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return DataCardRepository(mockedContext)
    }

    @Test
    fun getDataCards() = runBlocking {
        val repo = getRepo()

        repo.hasMultipleDataCards.observeForTesting {
            repo.hasMultipleDataCards.value.assertNull()
        }
        repo.userInfoFromDataCard.assertNull()

        repo.getDataCards(this)
            .await()

        repo.userInfoFromDataCard.assertNotNullAndApply {
            identifier.assertNotNullAndEquals("ID2")
        }

        repo.hasMultipleDataCards.observeForTesting {
            repo.hasMultipleDataCards.value.assertNotNullAndEquals(true)
        }

        repo.dataCardWarning.observeForTesting {
            repo.dataCardWarning.value.assertNull()
        }

        repo.currentDataCard.observeForTesting {
            repo.currentDataCard.value.assertNotNullAndApply {
                episodeOfCareIdentifier.assertEquals("EP2")
            }
        }

        getFailingRepo().getDataCards(this)
            .await()
            .assertIs<Failed>()
    }

    @Test
    fun updateDateOfBirthForCurrentSelection() = runBlocking {
        val repo = getRepo()

        repo.updateDateOfBirthForCurrentSelection(this, LocalDateTime.of(2000, 1, 1, 1, 1, 1))
            .await()
            .assertIs<Failed>("Can't set date of birth before actually having card")

        repo.getDataCards(this)
            .await()

        repo.dateOfBirthForCurrentDataCard.observeForTesting {
            repo.dateOfBirthForCurrentDataCard.value.assertEquals(
                LocalDateTime.of(
                    2018,
                    7,
                    6,
                    4,
                    1,
                ),
            )
        }

        repo.currentDataCard.observeForTesting {
            runBlocking {
                val result = repo.updateDateOfBirthForCurrentSelection(
                    this,
                    LocalDateTime.of(2021, 9, 2, 4, 3),
                ).await()

                result.assertIsAndApply<Success<DateOfBirth>> {
                    val successDateOfBirth = value
                    repo.dateOfBirthForCurrentDataCard.observeForTesting {
                        repo.dateOfBirthForCurrentDataCard.value.assertEquals(
                            successDateOfBirth.dateOfBirth,
                        )
                    }
                }
            }
        }

        getFailingRepo().updateDateOfBirthForCurrentSelection(
            this,
            LocalDateTime.of(2021, 9, 2, 4, 3),
        ).await()
            .assertIs<Failed>()
    }

    @Test
    fun clear() = runBlocking {
        val repo = getRepo()
        repo.getDataCards(this).await()

        repo.currentDataCard.observeForTesting {
            repo.currentDataCard.value.assertNotNull()
        }

        repo.updateDateOfBirthForCurrentSelection(
            this,
            LocalDateTime.of(2021, 9, 2, 4, 3),
        ).await()

        repo.dateOfBirthForCurrentDataCard.observeForTesting {
            repo.dateOfBirthForCurrentDataCard.value.assertNotNull()
        }

        repo.clear()

        repo.currentDataCard.observeForTesting {
            repo.currentDataCard.value.assertNull()
        }

        repo.dateOfBirthForCurrentDataCard.observeForTesting {
            repo.dateOfBirthForCurrentDataCard.value.assertNull()
        }
    }
}

/**
 * Observes a [LiveData] until the `block` is done executing.
 * This is required for LiveData that is created through Transformations or is never observed in the repo
 */
private fun <T> LiveData<T>.observeForTesting(block: () -> Unit) {
    val observer = Observer<T> { }
    try {
        observeForever(observer)
        block()
    } finally {
        removeObserver(observer)
    }
}
