/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.repository

import android.content.Context
import dk.nsp.api.IBackendApiLayer
import dk.nsp.api.observations.IObservationsController
import dk.nsp.models.ObservationTable
import dk.nsp.models.ObservationTableMother
import dk.nsp.models.ObservationTableMotherRows
import dk.nsp.models.ScanningObservationTable
import dk.nsp.models.ScanningObservationTableMother
import dk.nsp.models.ScanningObservationTableMotherRows
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.shared.toSuccess
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIs
import dk.nsp.testassertions.assertIsAndApply
import dk.nsp.testassertions.assertNotNullAndApply
import io.mockk.coEvery
import io.mockk.every
import io.mockk.mockk
import io.mockk.mockkObject
import kotlinx.coroutines.runBlocking
import org.junit.jupiter.api.Test

class ObservationsRepositoryTest {

    private val mockedController = mockk<IObservationsController> {
        coEvery {
            getScanObservationsTable(any(), any()).await()
        } returns ScanningObservationTable(
            ScanningObservationTableMother(
                listOf(
                    ScanningObservationTableMotherRows(
                        "ActivityIdScan",
                    ),
                ),
            ),
        ).toSuccess()

        coEvery {
            getClinicalObservationsTable(any(), any()).await()
        } returns ObservationTable(
            ObservationTableMother(
                listOf(
                    ObservationTableMotherRows(
                        "ActivityIdTable",
                    ),
                ),
            ),
        ).toSuccess()
    }

    private val failingController = mockk<IObservationsController> {
        coEvery {
            getScanObservationsTable(any(), any()).await()
        } returns Failed(Exception())

        coEvery {
            getClinicalObservationsTable(any(), any()).await()
        } returns Failed(Exception())
    }

    private val mockBackend = mockk<IBackendApiLayer> {
        every {
            observationsController
        } returns mockedController
    }

    private val failingBackend = mockk<IBackendApiLayer> {
        every {
            observationsController
        } returns failingController
    }

    private fun getRepo(): ObservationsRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns mockBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return ObservationsRepository(mockedContext)
    }

    private fun getFailingRepo(): ObservationsRepository {
        mockkObject(IBackendApiLayer)
        every {
            IBackendApiLayer.getLayer(any())
        } returns failingBackend

        val mockedContext = mockk<Context>(relaxed = true)

        return ObservationsRepository(mockedContext)
    }

    @Test
    fun getClinicalObservationsAsync() = runBlocking {
        getRepo().getClinicalObservationsAsync(
            this,
            "",
        ).await()
            .assertIsAndApply<Success<ObservationTable>> {
                value.mother.assertNotNullAndApply {
                    rows.assertNotNullAndApply {
                        getOrNull(0).assertNotNullAndApply {
                            activityId.assertEquals("ActivityIdTable")
                        }
                    }
                }
            }

        getFailingRepo().getClinicalObservationsAsync(
            this,
            "",
        ).await()
            .assertIs<Failed>()
    }

    @Test
    fun getScanObservationsTable() = runBlocking {
        getRepo().getScanObservationsTable(
            this,
            "",
        ).await()
            .assertIsAndApply<Success<ScanningObservationTable>> {
                value.mother.assertNotNullAndApply {
                    rows.assertNotNullAndApply {
                        getOrNull(0).assertNotNullAndApply {
                            activityId.assertEquals("ActivityIdScan")
                        }
                    }
                }
            }

        getFailingRepo().getScanObservationsTable(
            this,
            "",
        ).await()
            .assertIs<Failed>()
    }
}
