/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.util

import dk.nsp.models.CprIdentifiedPerson
import dk.nsp.models.DataCard
import dk.nsp.models.PersonName
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertFalse
import dk.nsp.testassertions.assertNull
import dk.nsp.testassertions.assertTrue
import org.junit.jupiter.api.Test
import java.time.LocalDateTime

class DataCardExtensionsTest {

    companion object {
        private val DataCardEmpty = DataCard(
            "",
            CprIdentifiedPerson(PersonName()),
        )
    }

    @Test
    fun dataCardIsEnded() {
        DataCardEmpty.copy(documentStatus = DataCard.DocumentStatus.ACTIVE).isEnded.assertFalse()
        DataCardEmpty.copy(documentStatus = DataCard.DocumentStatus.CLOSED).isEnded.assertTrue()
        DataCardEmpty.copy(documentStatus = DataCard.DocumentStatus.PARTIAL).isEnded.assertFalse()
        DataCardEmpty.copy(documentStatus = DataCard.DocumentStatus.NEW).isEnded.assertFalse()
    }

    @Test
    fun dataCardIsActive() {
        DataCardEmpty.copy(documentStatus = DataCard.DocumentStatus.ACTIVE).isActive.assertTrue()
        DataCardEmpty.copy(documentStatus = DataCard.DocumentStatus.CLOSED).isActive.assertFalse()
        DataCardEmpty.copy(documentStatus = DataCard.DocumentStatus.PARTIAL).isActive.assertFalse()
        DataCardEmpty.copy(documentStatus = DataCard.DocumentStatus.NEW).isActive.assertTrue()
    }

    class ListDataCardGetPreselectionOrMostRecent {
        @Test
        fun empty() {
            emptyList<DataCard>().getPreselectionOrMostRecent(null)
        }

        @Test
        fun withId() {
            val matching = DataCardEmpty.copy(
                episodeOfCareIdentifier = "ID1",
            )
            val activeButNotAMatch = DataCardEmpty.copy(
                episodeOfCareIdentifier = "ID0",
                documentStatus = DataCard.DocumentStatus.ACTIVE,
            )

            val items = listOf(
                activeButNotAMatch,
                matching,
            )

            items.getPreselectionOrMostRecent("ID1")
                .assertEquals(matching)

            // With no ID match, we should get the first Active (But this logic is tested in GetMostRecentDataCard)
            items.getPreselectionOrMostRecent("ID2")
                .assertEquals(activeButNotAMatch)
        }
    }

    class ListDataCardGetMostRecentDataCard {
        @Test
        fun empty() {
            emptyList<DataCard>().getMostRecentDataCard().assertNull()
        }

        @Test
        fun firstActive() {
            val activeCard = DataCardEmpty.copy(
                documentStatus = DataCard.DocumentStatus.ACTIVE,
            )

            listOf(
                DataCardEmpty.copy(
                    documentStatus = DataCard.DocumentStatus.CLOSED,
                ),
                activeCard,
            ).getMostRecentDataCard().assertEquals(activeCard)
        }

        @Test
        fun latestModified() {
            val oldCard = DataCardEmpty.copy(
                documentStatus = DataCard.DocumentStatus.CLOSED,
                modifiedTime = LocalDateTime.of(2021, 9, 8, 7, 6),
            )

            val futureCard = DataCardEmpty.copy(
                documentStatus = DataCard.DocumentStatus.CLOSED,
                modifiedTime = LocalDateTime.now().plusMinutes(1),
            )

            val nullModifiedCard = DataCardEmpty.copy(
                documentStatus = DataCard.DocumentStatus.CLOSED,
            )

            listOf(
                oldCard,
                nullModifiedCard,
            ).getMostRecentDataCard().assertEquals(nullModifiedCard)

            listOf(
                nullModifiedCard,
                futureCard,
            ).getMostRecentDataCard().assertEquals(futureCard)
        }
    }

    class ListDataCardOnlyHasPreviousCourses {
        @Test
        fun empty() {
            emptyList<DataCard>()
                .onlyHasPreviousCourses()
                .assertTrue()
        }

        @Test
        fun single() {
            listOf(
                DataCardEmpty.copy(
                    documentStatus = DataCard.DocumentStatus.ACTIVE,
                ),
            ).onlyHasPreviousCourses().assertFalse()

            listOf(
                DataCardEmpty.copy(
                    documentStatus = DataCard.DocumentStatus.CLOSED,
                ),
            ).onlyHasPreviousCourses().assertTrue()
        }

        @Test
        fun multiple() {
            listOf(
                DataCardEmpty.copy(
                    documentStatus = DataCard.DocumentStatus.ACTIVE,
                ),
                DataCardEmpty.copy(
                    documentStatus = DataCard.DocumentStatus.CLOSED,
                ),
            ).onlyHasPreviousCourses().assertFalse()

            listOf(
                DataCardEmpty.copy(
                    documentStatus = DataCard.DocumentStatus.PARTIAL,
                ),
                DataCardEmpty.copy(
                    documentStatus = DataCard.DocumentStatus.CLOSED,
                ),
            ).onlyHasPreviousCourses().assertTrue()
        }
    }
}
