/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
@file:Suppress(
    "ArrayInDataClass",
    "EnumEntryName",
    "RemoveRedundantQualifierName",
    "UnusedImport",
)

package dk.nsp.models

import android.os.Parcelable
import com.squareup.moshi.Json
import kotlinx.parcelize.Parcelize

/**
 *
 *
 * @param status
 * @param title
 * @param practitioner
 * @param location
 * @param scheduledString scheduledString from https://www.nspop.dk/display/public/web/GCP+-+Guide+til+anvendere
 * @param plannedTime
 * @param conductedTime This field is computed based on the service start time of X-referenced documents (measurements + notes)
 * @param ownership
 * @param description
 * @param reference A pointer to whatever object this activity is related to
 * @param code
 * @param kind
 * @param id
 * @param careplanId A unique pointer to the careplan document from which this appointment origins
 * @param note
 * @param observations
 * @param sortDate Since activities may or may not have an actual plannedTime on them, this field is a computed date based on planned plannedTime or conductednTime
 */
@Parcelize
data class Activity(

    @Json(name = "status")
    val status: Activity.Status,

    @Json(name = "title")
    val title: kotlin.String,

    @Json(name = "practitioner")
    val practitioner: ActivityPractitioner? = null,

    @Json(name = "location")
    val location: ActivityLocation? = null,

    // scheduledString from https://www.nspop.dk/display/public/web/GCP+-+Guide+til+anvendere
    @Json(name = "scheduledString")
    val scheduledString: kotlin.String? = null,

    @Json(name = "plannedTime")
    val plannedTime: ActivityPlannedTime? = null,

    // This field is computed based on the service start time of X-referenced documents (measurements + notes)
    @Json(name = "conductedTime")
    val conductedTime: java.time.LocalDateTime? = null,

    @Json(name = "ownership")
    val ownership: Activity.Ownership? = null,

    @Json(name = "description")
    val description: kotlin.String? = null,

    // A pointer to whatever object this activity is related to
    @Json(name = "reference")
    val reference: kotlin.String? = null,

    @Json(name = "code")
    val code: Code? = null,

    @Json(name = "kind")
    val kind: kotlin.String? = null,

    @Json(name = "id")
    val id: kotlin.String? = null,

    // A unique pointer to the careplan document from which this appointment origins
    @Json(name = "careplanId")
    val careplanId: kotlin.String? = null,

    @Json(name = "note")
    val note: Note? = null,

    @Json(name = "observations")
    val observations: kotlin.collections.List<Observation>? = null,

    // Since activities may or may not have an actual plannedTime on them, this field is a computed date based on planned plannedTime or conductednTime
    @Json(name = "sortDate")
    val sortDate: java.time.LocalDateTime? = null,

) : Parcelable {

    /**
     *
     *
     * Values: Expected,Planned,Finished,Cancelled
     */
    enum class Status(val value: kotlin.String) {
        @Json(name = "expected")
        Expected("expected"),

        @Json(name = "planned")
        Planned("planned"),

        @Json(name = "finished")
        Finished("finished"),

        @Json(name = "cancelled")
        Cancelled("cancelled"),
    }

    /**
     *
     *
     * Values: CurrentUser,OtherUser
     */
    enum class Ownership(val value: kotlin.String) {
        @Json(name = "currentUser")
        CurrentUser("currentUser"),

        @Json(name = "otherUser")
        OtherUser("otherUser"),
    }
}
