/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
@file:Suppress(
    "ArrayInDataClass",
    "EnumEntryName",
    "RemoveRedundantQualifierName",
    "UnusedImport",
)

package dk.nsp.models

import android.os.Parcelable
import com.squareup.moshi.Json
import kotlinx.parcelize.Parcelize

/**
 * A way to carry machine-readable details of errors in a HTTP response to avoid the need to define new error response formats for HTTP APIs. See [RFC7807](https://tools.ietf.org/rfc/rfc7807.txt)
 *
 * @param type A URI reference [RFC3986](https://tools.ietf.org/rfc/rfc3986.txt) that identifies the problem type.
 * @param title A short, human-readable summary of the problem type.
 * @param status The HTTP status code [([RFC7231], Section 6)](https://tools.ietf.org/rfc/rfc7231.txt#section-6) generated by the origin server for this occurrence of the problem.
 * @param instance A URI reference that identifies the specific occurrence of the problem.
 * @param detail A human-readable explanation specific to this occurrence of the problem.
 * @param debugging Debugging information for DEV and QA environments.
 * @param invalidParams An array of validation errors.
 */
@Parcelize
data class ProblemDetails(

    // A URI reference [RFC3986](https://tools.ietf.org/rfc/rfc3986.txt) that identifies the problem type.
    @Json(name = "type")
    val type: java.net.URI? = null,

    // A short, human-readable summary of the problem type.
    @Json(name = "title")
    val title: kotlin.String? = null,

    // The HTTP status code [([RFC7231], Section 6)](https://tools.ietf.org/rfc/rfc7231.txt#section-6) generated by the origin server for this occurrence of the problem.
    @Json(name = "status")
    val status: java.math.BigDecimal? = null,

    // A URI reference that identifies the specific occurrence of the problem.
    @Json(name = "instance")
    val instance: kotlin.String? = null,

    // A human-readable explanation specific to this occurrence of the problem.
    @Json(name = "detail")
    val detail: kotlin.String? = null,

    // Debugging information for DEV and QA environments.
    @Json(name = "debugging")
    val debugging: kotlin.String? = null,

    // An array of validation errors.
    @Json(name = "invalid-params")
    val invalidParams: kotlin.collections.Set<ProblemDetailsInvalidParams>? = null,

) : Parcelable
