/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.shared

import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertIs
import dk.nsp.testassertions.assertIsAndApply
import dk.nsp.testassertions.assertNotNullAndEquals
import dk.nsp.testassertions.assertNull
import org.junit.jupiter.api.Test

class ResultWrapperTest {
    @Test
    fun tToSuccess() {
        "string".toSuccess().assertIsAndApply<Success<String>> { value.assertEquals("string") }
        12.toSuccess().assertIsAndApply<Success<Int>> { value.assertEquals(12) }
    }

    @Test
    fun resultWrapperTMapSuccess() {
        val success = Success("ThisThing").mapSuccess {
            it.assertEquals("ThisThing")
            "$it edited"
        }
        success.assertIsAndApply<Success<String>> {
            value.assertEquals("ThisThing edited")
        }

        val failed = Failed(IllegalStateException("StateError")).mapSuccess {}
        failed.assertIsAndApply<Failed> {
            error.assertIs<IllegalStateException>()
            error.message.assertEquals("StateError")
        }
    }

    @Test
    fun resultWrapperTMapSuccessWithTryOrFailed() {
        val originalFailed = Failed(IllegalStateException("StateError"))
        val failed = originalFailed.mapSuccessWithTryOrFailed {}
        failed.assertEquals(originalFailed)
        failed.assertIsAndApply<Failed> {
            error.assertIs<IllegalStateException>()
            error.message.assertEquals("StateError")
        }

        val success = Success(123).mapSuccessWithTryOrFailed {
            it + 4000
        }
        success.assertIsAndApply<Success<Int>> {
            value.assertEquals(4123)
        }

        val throwingSuccess = Success(123).mapSuccessWithTryOrFailed {
            throw IllegalArgumentException("ArgumentError")
        }
        throwingSuccess.assertIsAndApply<Failed> {
            error.assertIs<IllegalArgumentException>()
            error.message.assertEquals("ArgumentError")
        }
    }

    @Test
    fun resultWrapperTAsSuccessValueOrNull() {
        val success = Success("resultValue").asSuccessValueOrNull()
        success.assertEquals("resultValue")

        val failed: ResultWrapper<String> = Failed(Exception("Error"))
        val failedResult = failed.asSuccessValueOrNull()
        failedResult.assertNull()
    }

    @Suppress("UNREACHABLE_CODE")
    @Test
    fun tryOrFailed() {
        tryOrFailed {
            42
        }.assertNotNullAndEquals(Success(42))

        val exception = RuntimeException("What?!")
        tryOrFailed {
            throw exception
        }.assertNotNullAndEquals(Failed(exception))

        val th = Throwable()
        tryOrFailed {
            throw th
        }.assertNotNullAndEquals(Failed(th))
    }

    @Test
    fun map() {
        val withResult = ResultWrapper.map("result", NoStackTraceException("Error"))
        withResult.assertIsAndApply<Success<String>> {
            value.assertEquals("result")
        }

        val withoutResult = ResultWrapper.map(null, IllegalArgumentException("ErrorError"))
        withoutResult.assertIsAndApply<Failed> {
            error.assertIs<IllegalArgumentException>()
            error.message.assertEquals("ErrorError")
        }
    }
}
