/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.shared.util.extensions

import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertFalse
import dk.nsp.testassertions.assertNull
import dk.nsp.testassertions.assertTrue
import dk.nsp.testassertions.testWithDanishLocale
import org.junit.jupiter.api.Test
import java.time.LocalDateTime

class LocalDateTimeExtensionsTest {

    class LocalDateTimeWeekNumberAfterBirthDate {
        @Test
        fun justBelowWeeksThreshold() {
            LocalDateTime.now()
                .minusWeeks(1)
                .minusMinutes(1)
                .weekNumberAfterBirthDate().assertEquals(2)

            LocalDateTime.now()
                .minusWeeks(2)
                .minusMinutes(1)
                .weekNumberAfterBirthDate().assertEquals(3)
        }

        @Test
        fun justAboveWeeksThreshold() {
            LocalDateTime.now()
                .minusWeeks(1)
                .plusMinutes(1)
                .weekNumberAfterBirthDate().assertEquals(1)

            LocalDateTime.now()
                .minusWeeks(2)
                .plusMinutes(1)
                .weekNumberAfterBirthDate().assertEquals(2)
        }

        @Test
        fun basic() {
            LocalDateTime.now()
                .minusDays(3)
                .weekNumberAfterBirthDate().assertEquals(1)

            LocalDateTime.now()
                .minusDays(10)
                .weekNumberAfterBirthDate().assertEquals(2)
        }
    }

    @Test
    fun localDateTimeToDefaultMonthDisplay() {
        testWithDanishLocale {
            LocalDateTime.of(2021, 10, 2, 3, 4, 5)
                .toDefaultMonthDisplay()
                .assertEquals("2. okt. 2021")
        }
    }

    @Test
    fun localDateTimeToPrettyString() {
        testWithDanishLocale {
            LocalDateTime.of(2021, 1, 2, 3, 4, 5)
                .toPrettyString()
                .assertEquals("2. januar 2021 kl. 03:04")
        }
    }

    @Test
    fun localDateTimeToMediumFormat() {
        LocalDateTime.of(2021, 1, 2, 3, 4, 5)
            .toMediumFormat()
            .assertEquals("02.01.2021")

        val nullable: LocalDateTime? = null
        nullable.toMediumFormat().assertNull()
    }

    @Test
    fun localDateTimeToLongFormat() {
        testWithDanishLocale {
            LocalDateTime.of(2021, 5, 15, 3, 4, 5)
                .toLongFormat()
                .assertEquals("Lørdag, 15. maj")

            val nullable: LocalDateTime? = null
            nullable.toLongFormat().assertEquals("")
        }
    }

    @Test
    fun localDateTimeToFullDateFormat() {
        testWithDanishLocale {
            LocalDateTime.of(2021, 7, 27, 3, 4, 5)
                .toFullDateFormat()
                .assertEquals("Tirsdag, 27. juli 2021")

            val nullable: LocalDateTime? = null
            nullable.toFullDateFormat().assertEquals("")
        }
    }

    @Test
    fun localDateTimeToTimeFormat() {
        LocalDateTime.of(2021, 1, 2, 3, 4, 5)
            .toTimeFormat()
            .assertEquals("03:04")
    }

    @Test
    fun localDateTimeToLongMonthFormat() {
        testWithDanishLocale {
            LocalDateTime.of(2021, 3, 2, 3, 4, 5)
                .toLongMonthFormat()
                .assertEquals("Marts")
        }
    }

    @Test
    fun localDateTimeWithTimeAtStartOfDay() {
        LocalDateTime.of(2021, 1, 2, 3, 4, 5)
            .withTimeAtStartOfDay()
            .assertEquals(LocalDateTime.of(2021, 1, 2, 0, 0, 0))
    }

    @Test
    fun localDateTimeIsBeforeNow() {
        LocalDateTime.now().minusSeconds(1)
            .isBeforeNow()
            .assertTrue()

        LocalDateTime.now().plusSeconds(1)
            .isBeforeNow()
            .assertFalse()

        val nullable: LocalDateTime? = null
        nullable.isBeforeNow().assertFalse()
    }

    @Test
    fun localDateTimeIsAfterNow() {
        LocalDateTime.now().plusSeconds(1)
            .isAfterNow()
            .assertTrue()

        LocalDateTime.now().minusSeconds(1)
            .isAfterNow()
            .assertFalse()

        val nullable: LocalDateTime? = null
        nullable.isAfterNow().assertFalse()
    }

    @Test
    fun localDateTimeIsBeforeNowBySeconds() {
        LocalDateTime.now().minusSeconds(5)
            .isBeforeNow(4)
            .assertTrue()

        LocalDateTime.now().minusSeconds(5)
            .isBeforeNow(6)
            .assertFalse()

        LocalDateTime.now().plusSeconds(1)
            .isBeforeNow(0)
            .assertFalse()

        val nullable: LocalDateTime? = null
        nullable.isBeforeNow(3).assertTrue()
    }
}
